#!/usr/bin/perl -w
# ^--- change this if perl is installed in a different path

##############################################################################
#                                                                            #
#  OpenFTPD Installscript v1.2 (c) 2000 primemover                           #
#                                                                            #
# ---------------------------------------------------------------------------#
#                                                                            #
#  This program is free software; you can redistribute it and/or modify      #
#  it under the terms of the GNU General Public License as published by      #
#  the Free Software Foundation; either version 2 of the License, or         #
#  (at your option) any later version.                                       #
#                                                                            #
##############################################################################

use FindBin;
use lib "$FindBin::Bin/lib";
use ANSIColor;
use Term::ReadLine;

sub bold { print $OUT color("bold"); }
sub normal { print $OUT color("reset"); }


sub expand {
	my $string = shift;
	$string =~ s/\$HOME/$ENV{HOME}/i;
	$string =~ s{ ^ ~ ( [^/]* ) }  { 
		$1 ? (getpwnam($1))[7] : 
	        ($ENV{HOME} || $ENV{LOGDIR} || (getpwuid($<))[7])
	}ex;
	return $string;
}

sub update {
	normal();
	print $OUT <<'EOT';

Please enter the path to your OpenFTPD directory (where sbin/ is located but 
dont add "sbin/" to the path) Example: "/home/foo/openftpd" 

EOT

	$installdir = "";
	bold();
	do {
		$installdir = $term->readline("OpenFTPD directory: ");
		$installdir = expand($installdir);
	} until -d "$installdir/sbin";
	chomp($installdir);
	normal();

	# kill running ftpd
	if (open (PIDFILE, "< $installdir/var/ftpd.pid")) {
		print $OUT "\nTrying to shutdown ftpd ...\n";
		my $pid = <PIDFILE>;
		chomp($pid);
		if ($pid) {		
			if (kill TERM => $pid) {
				print $OUT "The ftpd with pid $pid was killed.\n";
			}
		}
	}

	# copy files
	print $OUT "\nUpdating binaries ...\n\n";
	system("cp -f build/ftpd/ftpd $installdir/sbin/");
	system("cp -f build/ftps/ftps $installdir/sbin/");
	system("cp -f build/ftpa/ftpa $installdir/sbin/");
	system("cp -f build/misc/checksum $installdir/bin/");
	system("cp -f build/misc/getpw $installdir/bin/");
	system("cp -f build/misc/glconv $installdir/bin/");
	system("cp -f build/misc/msg $installdir/bin/");
	system("cp -f standard/sbin/ftps.pl $installdir/sbin/");

	bold();
	my $key = 0;
	while ($key !~ /^(y|n)$/) {
		$key = $term->readline("Update (OVERWRITE!) scripts too ? (y/n) : " );
		chomp($key);
	}
	normal();
	if ($key eq 'y') {
		print $OUT "\nUpdating scripts ...\n\n";
		system("cp -f standard/scripts/*.pl $installdir/scripts/");
	}

	bold();
	$key = 'n';
	$key = $term->readline("Restart OpenFTPD ? (y/n) : " );
	chomp($key);
	normal();

	print $OUT "\nOpenFTPD was updated.\n\n";
	if ($key eq 'y') {
		system("$installdir/sbin/ftpd");
	}

	return 0;
}


sub install {
	normal();
	print $OUT <<'EOT';

First you have to select the basepath for the ftpd itself and it's
configuration, log and data files. Don't mix up with the ftp root path
which you'll set later. Use an existing path with write permissions
for your account. 

The default path is ~/openftpd (press ENTER to use this). 
Example: /home/foo/openftpd

EOT

	$installdir = "";
	bold();
	$installdir = $term->readline("OpenFTPD directory: ");
	chomp($installdir);
	normal();

	$installdir = "$ENV{HOME}/openftpd" unless $installdir;

	# expand the string
	$installdir =~ s/\$HOME/$ENV{HOME}/i;
	$installdir =~ s{ ^ ~ ( [^/]* ) }  { 
		$1 ? (getpwnam($1))[7] : 
	        ($ENV{HOME} || $ENV{LOGDIR} || (getpwuid($<))[7])
	}ex;


	bold();
	my $key = 0;
	while ($key !~ /^(y|n)$/) {
		$key = $term->readline("Install OpenFTPD to $installdir ? (y/n): " );
		chomp($key);
	}
	normal();

	if ($key eq "n") {
		print "Installation aborted.\n\n"; 
		return -1;
	}

	if (not -d $installdir) { 
		mkdir($installdir, 0700); 
	}

	$rc = system("cp -R standard/* $installdir/");
	if ($rc) {
		print $OUT "\nUnable to copy files. Check your permissions!\n";
		normal();
   		return -1;
	}

	# copy files
	system("cp -f build/ftpd/ftpd $installdir/sbin/");
	system("cp -f build/ftps/ftps $installdir/sbin/");
	system("cp -f build/ftpa/ftpa $installdir/sbin/");
	system("cp -f build/misc/checksum $installdir/bin/");
	system("cp -f build/misc/getpw $installdir/bin/");
	system("cp -f build/misc/glconv $installdir/bin/");
	system("cp -f build/misc/msg $installdir/bin/");

	print $OUT "\nWriting install path to ~/.openftpdrc ...\n";
	open (OPENFTPDRC, "> $ENV{'HOME'}/.openftpdrc");
	printf OPENFTPDRC "%s/", $installdir;
	close(OPENFTPDRC);

	print $OUT "\nFiles were successfully installed to $installdir.\n";

	config();
	return 0;
}


sub config {
	normal();
	print $OUT <<'EOT';

You should now enter the path for your FTP base directory, the root path
that the FTP users will see. 

EOT

	my $basedir;
	bold();
	while (!$basedir) {
		$basedir = $term->readline("FTP root directory: ");
		$basedir = expand($basedir);
	}
	chomp($basedir);
	normal();
	
	if (not -d $basedir) {
		if (mkdir($basedir, 0700)) {
			print $OUT "$basedir was successfully created.\n";
		}
		else { 
			die "ERROR: unable to create $basedir\n"; 
		}
	}
	print $OUT "\nOpenFTPD is using $basedir as FTP base directory.\n";

print $OUT <<'EOT';

Please select the ports for FTP and admin GUI (not yet finished) connections.
Choose ports between 1024 and 65536 if you are not root.

EOT
	my $port = 0;
	
	bold();
	while($port !~ /^\d{2,5}$/) {
		$port = $term->readline("FTP port: ");
		chomp($port);
	}
	normal();

	$adminport = 0;
	bold();
	while($adminport !~ /^\d{2,5}$/) {
		$adminport = $term->readline("Admin port: ");
		chomp($port);
	}
	normal();
	print $OUT <<'EOT';

Please enter a short name (about 3 letters) for your site, which will be used
for by several addons.

EOT

	$shortsitename = "";
	bold();
	while ($shortsitename !~ /.+/) {
   		$shortsitename = $term->readline("Short name of the site: ");
   		chomp($shortsitename);
	}
	normal();
	
	chmod 0640, "$installdir/etc/ftpd.reg";
	open(F, "> $installdir/etc/ftpd.reg") 
		or die "ERROR: unable to open $installdir/etc/ftpd.reg\n";

	print F 
		"port                 $port\n",
		"adminport            $adminport\n",
		"basedir              $basedir\n",
		"shortsitename        $shortsitename\n",
		"hiddenfiles          .dircfg .filled .locked .status\n",
		"readmefile           .readme\n", 
		"statdir              var/stats\n", 
		"loginkick            on\n", 
		"loglevel             3\n", 
		"ident                on\n", 
		"min_freemb           21\n",
		"private              on\n", 
		"checktimeout         15\n", 
		"chmod_allowed        no\n", 
		"filenameconversion   0x75\n", 
		"ffpath               /\n",
		"reject_hops          microsoft.com fbi.gov\n",
		"logusers             yes\n",
		"loggroups            yes\n",
		"timeoutuser          300\n",
		"timeoutroot          3600\n",
		"showinfo             yes\n";
	close(F);


	normal();
	print $OUT <<"EOT";

             _____
            /  _)))
           (___|''-                    INSTALLATION COMPLETE!
             ; _=
          ___//_   /_    _________     OpenFTPD was successfully installed
         /)  \\/ )  ))   |.        |_   on your system.
        //| - -/\\\\/;    |.        |:|
       |/ |   /  \\/     |.        |/   You may now start the ftpd with ...
       ;  :::::         |_________|    > $installdir/sbin/ftpd
     _(/ //////\\\\\\\\\\     __|___|__
  ___/|_//////// / /____[_________]_   ... and login (user root, no password)
               |/|/           b'ger    > ftp localhost $port
               | |
              (|(|                     Don't forget the read the included
             ,||||                     manuals and checkout www.openftpd.org
              '='=                     where you can find most information.

                                       HAVE FUN!
 
EOT
}


# main program

$term = new Term::ReadLine 'OpenFTPD Installer';
$OUT = $term->OUT || STDOUT;

#system("clear");
#normal();

bold();
print $OUT "OpenFTPD Installer v1.5 (c) 2001 Primemover / Team OpenFTPD\n";
normal();
normal();

print $OUT <<'EOT';


 ___.___    ~            _____________
 \  \\  \   ,, ???      |        '\\\\\\     Welcome to the interactive
  \  \\  \ /<   ?       |        ' ____|_    OpenFTPD installation program.
   --\//,- \_.  /_____  |        '||::::::
       o- /   \_/    '\ |        '||_____|   This program will install all
       | \ '   o       \'________|_____|     required files and will guide you
       |  )-   #     <  ___/____|___\___     through the basic configuration
       `_/'------------|    _    '  <<<:|    of OpenFTPD
  b'ger    /________\| |_________'___o_o| __________________________________ _


EOT

if ((not -e "build/ftpd/ftpd") or (not -e "build/ftps/ftps")) {
	bold();
	print "WARNING: YOU HAVE TO COMPILE OPENFTPD BEFORE INSTALLATION!\n";
	print "         READ THE FILES 'README' AND 'INSTALL' !!!! \n";
	normal();
	exit(-1);
}


bold();
print "Do you want a fresh installation or to update an installed system ?\n\n";
normal();
      

bold();
$installmode = "";
while ($installmode !~ /^(n|u)$/) {
	$installmode = $term->readline("Install mode (n/u): ");
	chomp($installmode);
}
normal();

if ($installmode eq 'n') {
	install();
}
else {
	update();
}


# --- end of file ---
