/*
	$Id: site_vfs.h,v 1.4 2001/02/27 00:44:32 hvr Exp $

	Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __SITE_VFS_H__
#define __SITE_VFS_H__
#include <glib.h>

/* #define SITE_VFS_EXPERIMENTAL */

typedef enum {
  VFS_OK    = 0,    /* nothing */
  VFS_UNKOWN,       /* generic/unknown error */
  VFS_ACCESS,       /* permission denied */
  VFS_NOENT,        /* entry not found */
  VFS_EXIST,        /* entry exists already */
  VFS_NOTDIR,       /* not a directory */
  VFS_INVAL,        /* invalid arguments */
  /* ... */
} vfs_errno_t;

extern vfs_errno_t vfs_errno;

const gchar *vfs_strerror (vfs_errno_t err);

/* init */
void vfs_init (const gchar real_path[]);

/* replace ugly home lock hack 
   -- though it's not a real chroot() */
gint vfs_chroot (const gchar path[]);

/* godmode */
void vfs_set_superuser (gboolean set);

/* directory */
gint vfs_mkdir (const gchar path[]);
gint  vfs_setcwd (const gchar new_path[]);
const gchar *vfs_getcwd (void);

typedef enum {
  VFS_REG = 1,
  VFS_DIR,
  VFS_LNK /* not supported yet */
} vfs_type_t;

typedef enum {
  VFS_OTHER = 0,
  VFS_GROUP = 8,
  VFS_USER = 16,
  VFS_FLAG = 24
} vfs_perm_part_t;

guint32 vfs_perm_get (guint32 mode, vfs_perm_part_t part);
guint32 vfs_perm_set (guint32 mode, vfs_perm_part_t part, guint32 perm);

/* directory permissions */
#define VFS_DP_CHDIR     0x01
#define VFS_DP_LIST      0x02
#define VFS_DP_MKDIR     0x04
#define VFS_DP_RMDIR     0x08
#define VFS_DP_READ      0x10
#define VFS_DP_WRITE     0x20
#define VFS_DP_OVERWRITE 0x40
#define VFS_DP_CHMOD     0x80
#define VFS_DP_ALL       0xFF

/* directory flag */
#define VFS_DF_HIDDEN    0x01
#define VFS_DF_HIDDEN_R  0x10 /* virtual flag */

/* file permissions */
#define VFS_FP_WRITE     0x02
#define VFS_FP_READ      0x04
#define VFS_FP_CHMOD     0x08
#define VFS_FP_ALL       0x0E

/* warning -- some meanings differ from unix' struct stat!! */
typedef struct
{
  vfs_type_t type;

  /* all types */
  guint32 uid;
  guint32 gid;
  guint32 mode; /* aka permissions */
  gboolean cdpath_flag; /* if path is in cdpath db */

  guint32 create_time; /* file creation time */
  guint32 change_time; /* last change time */
  guint32 access_time; /* last access time (unused yet) */

  /* VFS_REG */
  guint64 size;
  guint32 crc; 
  guint32 dlc; /* access count */
} vfs_stat_t;

gint vfs_stat (const gchar pathname[], vfs_stat_t *buf);

gchar **vfs_readdir (const gchar path[]);

typedef enum {
  VFS_ATTR_MODE = 1,
  VFS_ATTR_UID,
  VFS_ATTR_GID,
  VFS_ATTR_CRC
} vfs_chattr_t;

gint vfs_chattr (const gchar pathname[], vfs_chattr_t attr, guint32 val);


/* ...may get replaced in future */
gchar *vfs_get_realpath (const gchar pathname[]);

#endif /* __SITE_VFS_H__ */

/* 
 * Local variables:
 *	c-file-style: "gnu"
 *	tab-width: 4
 *	indent-tabs-mode: t
 * End:
 */
