/* ---------------------------------------------------------------------------
 *
 *   This file is part of OpenFTPD (based on FTP4ALL)
 *   (C) 1996-2000 by Crescent, (C) 2000 by the OpenFTPD Team
 *
 * ---------------------------------------------------------------------------
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 * ---------------------------------------------------------------------------
 *   MODULE: log.c (ftpd) - fuctions for logging
 * ---------------------------------------------------------------------------
 */

#include <sys/types.h>
#include <errno.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
#include "externsd.h"
#include "f4adp_admin.h"
#include "registry.h"
#include "serverd.h"
#include "utils.h"
#include "strlcpy.h"
#include "strlcat.h"

int is_log(int l) 
{
   /* check current loglevel */
   if (l == LG_SECURITY && log_level < 1) return 0;
   if (l == LG_TRANSFER && log_level < 2) return 0;
   if ((l == LG_FILE || l == LG_DIRECTORY) && log_level < 2) return 0;
   if (l == LG_MISC && log_level < 2) return 0;
   if (l == LG_VERBOSE && log_level < 3) return 0;
   return 1;
}

void errlog(char* fmt, ...)
{ 
   va_list ap;
   char    msg[1024];
   time_t  t;

   strlcpy(msg, "+E ", sizeof(msg));
   time(&t);
   strftime(msg+3,64,"%a %d/%m/%Y %H:%M:%S",localtime(&t));
   g_snprintf(msg + strlen(msg), sizeof(msg) - strlen(msg), " [%ld] ", (long)getpid());
   va_start(ap,fmt);
   g_vsnprintf(msg + strlen(msg), sizeof(msg) - strlen(msg), fmt, ap);
   va_end(ap);
   if (errlogf) fprintf(errlogf,"%s\n",msg+3);
   notify_watchers(msg,0,NULL);
}


void errlog2(char* fmt, ...)
{ 
   va_list ap;
   char    msg[1024];
   time_t  t;

   strlcpy(msg, "+E ", sizeof(msg));
   time(&t);
   strftime(msg+3,64,"%a %d/%m/%Y %H:%M:%S",localtime(&t));
   g_snprintf(msg + strlen(msg), sizeof(msg) - strlen(msg), " [%ld] ", (long)getpid());
   va_start(ap,fmt);
   g_vsnprintf(msg + strlen(msg), sizeof(msg) - strlen(msg), fmt, ap);
   va_end(ap);
   fprintf(stderr,"%s\n",msg+3);
   fflush(stderr);
}


void open_logfiles(void)
{ 
   int   n;
   char* param[16];
   int   pipefd[2];
   char  tmpbuf[256];

   if (log_pid) { 
      fclose(log_program_f);
      log_program_f = NULL;
      log_pid = 0;
   }
   mk_filename(tmpbuf, errlog_file, DFL_ERRLOGFILE);
   errlogf = freopen(tmpbuf, "at", errlogf ? errlogf : stdout);
   if (errlogf) setvbuf(errlogf, NULL, _IOLBF, 0); 
   else printf("ftpd[%ld] freopen(%s): %s (%i)\n", (long)getpid(), tmpbuf, 
      strerror(errno), errno);
   setvbuf(stderr, NULL, _IOLBF,0);
   mk_filename(tmpbuf, ftpserrlog_file, DFL_FTPSERRLOGFILE);
   if (!freopen(tmpbuf, "at", stderr)) errlog("freopen(%s): %s (%i)\n",
      tmpbuf,strerror(errno),errno);
   if (logf) { 
      fclose(logf);
      logf = NULL;
   }
   mk_filename(tmpbuf, log_file, DFL_LOGFILE);
   if ((logf = fopen(tmpbuf, "at")) == NULL) {
      errlog("cannot open logfile (%s)", tmpbuf);
   }
   if (logf) { 
      if (fcntl(fileno(logf), F_SETFD, 1) < 0) {
         errlog("fcntl: %s (%i)",strerror(errno),errno);
      }
      setvbuf(logf, NULL, _IOLBF, 0);
   }
   if (log_program) { 
      if (logf) fflush(logf);
      if (errlogf) fflush(errlogf);
      fflush(stderr);
      if (pipe(pipefd) < 0) errlog("pipe(): %s (%i)",strerror(errno),errno);
      else {
	 switch (log_pid = fork()) { 
	    case  0 :
               close(0);
               close(1);
               close(2);
	       close(pipefd[1]);
	       if (dup2(pipefd[0],0) != -1) close(pipefd[0]);
	       n = 0;
	       parse_command(log_program,param,15,16,&n);
	       param[n] = NULL;
	       n = execvp(param[0],param);
	       exit(n);
	    case -1 :
	       errlog("fork: %s",strerror(errno));
	       log_pid = 0;
	       close(pipefd[0]);
	       close(pipefd[1]);
	       break;
	    default :
	       close(pipefd[0]);
	       log_program_f = fdopen(pipefd[1],"wt");
	       if (!log_program_f) { 
	          errlog("fdopen(%i): %s (%i)",pipefd[1],strerror(errno),errno);
		  close(pipefd[1]);
               }
	 }
      }
   }
}


void server_log_local(struct child_s* c, long l, char* str, ...)
{ 
   va_list ap;
   time_t  t;
   char    tstr[32];

   if (!logf || (is_log(l) == 0)) return;
   time(&t);
   strftime(tstr, sizeof(tstr), "%a %d/%m/%Y %H:%M:%S", localtime(&t));
   fprintf(logf, "%s %5ld %-9s ",tstr,c ? (long)c->pid : (long)getpid(),
      c && c->user ? c->user->name : "???");
   va_start(ap,str);
   vfprintf(logf,str,ap);
   fprintf(logf,"\n");
   if (log_pid) { 
      fprintf(log_program_f,"%s %5ld %-9s ",tstr,c ? (long)c->pid : (long)getpid(),
         c && c->user ? c->user->name : "???");
      vfprintf(log_program_f, str,ap);
      fprintf(log_program_f, "\n");
      fflush(log_program_f);
   }
}


void client_log(struct child_s* c, int idle, char** p, int n)
{ 
   int    i;
   int    l;
   char   tstr[32];
   char** q;

   if (!sscanf(p[0], "%i", &c->hidden)) c->hidden = 0;
   if (!sscanf(p[1], "%i", &l)) l = 0;
   c->idle = idle;

   if ((is_log(l) == 1) && (logf || log_program_f)) { 
      strftime(tstr,sizeof(tstr),"%a %d/%m/%Y %H:%M:%S",localtime(&c->act_time));
      if (logf) fprintf(logf,"%s %5ld %-9s",tstr,(long)c->pid,
 	 c->user ? c->user->name : "???");
      if (log_program_f) fprintf(log_program_f,"%s %5ld %-9s",tstr,(long)c->pid,
	 c->user ? c->user->name : "???");
      i = n - 2;
      q = p + 2;
      while (i--) { 
         if (logf) fprintf(logf, " %s", *q);
         if (log_program_f) fprintf(log_program_f, " %s", *q);
         q++;
      }
      if (logf) fprintf(logf, "\n");
      if (log_program_f) { 
         fprintf(log_program_f,"\n");
         fflush(log_program_f);
      }
   }
   /* client activity */
   memset(c->activity, 0, sizeof(c->activity));
   i = n - 2;
   q = p + 2;
   while (i--) { 
      if (strlen(c->activity) + strlen(*q)+1 < sizeof(c->activity)) { 
         if (*c->activity) strncat(c->activity, " ", sizeof(c->activity) - strlen(c->activity) - 1);
         strncat(c->activity, *q, sizeof(c->activity) - strlen(c->activity) - 1);
      } 
      else { 
         if (strlen(c->activity) < sizeof(c->activity)-3) { 
	    strlcat(c->activity, " ", sizeof(c->activity));
            strlcat(c->activity, *q, sizeof(c->activity));
	/*
            strlcpy(c->activity+strlen(c->activity), *q,
               sizeof(c->activity)-strlen(c->activity));
               *(c->activity+sizeof(c->activity)-1) = '\0';
	*/
         }
         i = 0;
      }
      q++;
   }
   server_reply(c, "+ok\n");
   notify_watchers_activity(c);
}


int f4adp_log_begin(struct child_s* c, char** p, int n)
{ 
   client_log(c, 0, p, n);
   return 0;
}


int f4adp_log_end(struct child_s* c, char** p, int n)
{ 
   client_log(c, 1, p, n);
   return 0;
}

/* EOF */
