/*   user.c
 *
 *   This file is part of FTP4ALL
 *
 *   Copyright (C) 1996-1999 by Crescent
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sys/types.h>
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdlib.h>
#ifdef DES_ENCRYPTION
  #include "des.h"
#endif
#include "externsd.h"
#include "log.h"
#include "utils.h"
#include "sections.h"
#include "strlcpy.h"
#include "strlcat.h"

int user_dirty = 0;
#ifdef DES_ENCRYPTION
  extern C_Block user_key;
  void   des_set_key(C_Block*,Key_schedule*);
  int    des_cbc_encrypt(C_Block*,C_Block*,int,Key_schedule*,C_Block*,int);
#endif
  
int get_ident(char** ipr, char* ident)
{ char* p;
  int	i;
  
  p = *ipr;
  i = 0;

  /* remove the ident if there is no @ in the whole ipstring */
  if (strchr(*ipr,'@') == NULL)
  { ident[0]='\0';
    return 0;
  }
  while (*p && (*p!='@') && (*p!='.') && (*p!=',') && (*p!=':')) {i++; p++; }
  if (!i) return -1;
  if (*p=='@')
  { *ipr=p+1;
    if (i>11) {p=p-i+11; i=11;}
    *p='\0';
    if (!sscanf(p-i,"%s",ident)) return -1;
  }
  if (ident[0]=='*') ident[0]='\0'; /* we dont need *@, so we remove the ident ... */
  return 0;
}

int get_oneip(char** ipr, char* l, char* h)
{ char  c;
  int   i;
  char* p;

  p = *ipr;
  if (!skipspace(&p)) return -1;
  if (*p == '*')
  { *l = 0x00;
    *h = 0xFF;
    p++;
    skipspace(&p);
    *ipr = p;
    return 0;
  }
  i = 0;
  while (*p && isdigit(*p) && (i<3)) { i++; p++; }
  if (!i) return -1;
  c = *p;
  *p = '\0';
  if (!sscanf(p-i,"%i",&i)) return -1;
  if ((i<0) || (i>255)) return -1;
  *l = i;
  *p = c;
  if (skipspace(&p) != '-')
  { *h = i;
    *ipr = p;
    return 0;
  }
  p++;
  if (!skipspace(&p)) return -1;
  i = 0;
  while (*p && isdigit(*p) && (i<3)) { i++; p++; }
  if (!i || isdigit(*p)) return -1;
  c = *p;
  *p = '\0';
  if (!sscanf(p-i,"%i",&i)) return -1;
  if ((i<0) || (i>255)) return -1;
  *p = c;
  *h = i;
  skipspace(&p);
  *ipr = p;
  return 0;
}

int get_iprange(char** ipr, char* ident)
{ if (get_ident(ipr, ident) == -1) return -1;
  if (get_oneip(ipr,&ident[12],&ident[13]) == -1) return -1;
  if (**ipr != '.') return -1;
  (*ipr)++;
  if (get_oneip(ipr,&ident[14],&ident[15]) == -1) return -1;
  if (**ipr != '.') return -1;
  (*ipr)++;
  if (get_oneip(ipr,&ident[16],&ident[17]) == -1) return -1;
  if (**ipr != '.') return -1;
  (*ipr)++;
  if (get_oneip(ipr,&ident[18],&ident[19]) == -1) return -1;
  return 0;
}

void inituser(struct user_s* u)
{ 
   if (u->limit) { 
      u->credit = (u->ul_total * 100 / u->limit) - u->dl_total;
   } 
   else u->credit = 0;
   memset(u->gids, -1, sizeof(u->gids));
   if (!u->uid) u->class = UC_ROOT; 
   else if (!strcasecmp(u->name, "anonymous") || !strcasecmp(u->name, "ftp")) {
      u->class = UC_ANON; 
   }
   else u->class = 0x10 | UC_NORMAL;
}

/* updates pointers in child structs */
void update_user(struct user_s* u_old, struct user_s* u_new)
{ 
   struct child_s* c;
   int             i;

   if (u_old != u_new) {
      for (i = 0, c = child; i < children; i++, c++) {
         if (c->user) {
            if (c->user >= u_old) c->user = u_new + (c->user - u_old);
	 }
      }
   }	    
}


/* insert user, keeping list sorted by uid */
struct user_s* insert_user(struct user_s* new)
{ 
   struct child_s*   c;
   int               i = 0;
   int               j;
   int               k = user_count - 1;
   struct user_s*    u;

   if (user_count >= user_size) { 
      user_size += 16;
      u = (struct user_s*)g_realloc(user, user_size * sizeof(struct user_s));

      if (!u) { 
         user_size -= 16;
         return NULL;
      }
      update_user(user, u);
      user = u;
   }
   while (i <= k) { 
      j = (i + k) >> 1;
      if (new->uid >= (user+j)->uid) i = j + 1; 
      else k = j - 1;
   }
   k++;
   if (k < user_count) { 
      memmove(user+k+1,user+k,(user_count-k)*sizeof(*u));
      for (j=0, c=child; j<children; j++, c++) {
         if (c->user && (c->user != (struct user_s*)-1)) {
            if (c->user-user >= k) c->user++;      
         }
      }      
   }
   *(user+k) = *new;
   user_count++;
   user_dirty++;
   return user+k;
}



int get_user(char* p)
{ char          c;
  int           i;
  char*         q;
  struct user_s usr;

  skipspace(&p);
  q = p;
  if (!skiptocolon(&q)) return -1;
  if (q-p<1 || q-p>=sizeof(usr.name)) return -1;
  strncpy(usr.name,p,q-p);
  *(usr.name+(q-p)) = '\0';
  if (!skipspctocolon(&q)) return -1;
  p = q;
  if (!skiptocolon(&q)) return -1;
  c  = *q;
  *q = '\0';
  if (!sscanf(p,SI32,&usr.uid)) return -1;
  *q = c;
  if (!skipspctocolon(&q)) return -1;
  p = q;
  if (!skiptocolon(&q)) return -1;
  c  = *q;
  *q = '\0';
  if (!sscanf(p,SI32,&usr.gid)) return -1;
  *q = c;
  if (!skipspctocolon(&q)) return -1;
  p = q;
  if (!skiptocolon(&q)) return -1;
  if (q-p>=sizeof(usr.password)) return -1;
  strncpy(usr.password,p,q-p);
  *(usr.password+(q-p)) = '\0';
  if (!skipspctocolon(&q)) return -1;
  p = q;
  if (!skiptocolon(&q)) return -1;
  c  = *q;
  *q = '\0';
  if (!sscanf(p,SI64,&usr.dl_total)) return -1;
  usr.dl_total *= 1024;
  *q = c;
  if (!skipspctocolon(&q)) return -1;
  p = q;
  skiptocolon(&q);
  c  = *q;
  *q = '\0';
  if (!sscanf(p,SI64,&usr.ul_total)) return -1;
  usr.ul_total *= 1024;
  *q = c;
  if (!skipspctocolon(&q)) return -1;
  p = q;
  skiptocolon(&q);
  c  = *q;
  *q = '\0';
  if (!sscanf(p, SITIME, &usr.last_login)) return -1;
  *q = c;
  if (!skipspctocolon(&q)) return -1;
  for (i=0;i<10;i++) memset(usr.idents[i],'\0',20);
  i = 0;
  while (i < 10)
  { if (get_iprange(&q,usr.idents[i]) == -1) return -1;
    skipspace(&q);
    if (*q == ',')
    { q++;
      if (++i == 10 ) return -1;
    } else i = 10;
  }
  if (!skipspctocolon(&q)) return -1;
  p = q;
  c = skiptocolon(&q);
  *q = '\0';
  if (!sscanf(p,SI32,&usr.limit)) return -1;
  *q = c;
  if (skipspctocolon(&q))
  { p = q;
    c = skiptocolon(&q);
    *q = '\0';
    if (strlen(p) >= sizeof(usr.info)) return -1;
    strlcpy(usr.info, p, sizeof(usr.info));
    *q = c;
    if (skipspctocolon(&q))
    { p = q;
      skiptocolon(&q);
      c  = *q;
      *q = '\0';
      if (!sscanf(p,SI32,&usr.class)) return -1;
      /*** class has changed a little bit ***/
      i = usr.class & 0x0F;         /* user class */
      i |= (usr.class & 0x70) << 4; /* logins */
      if (usr.class & 0x80) i |= UF_COLOR;
      if (usr.class & 0x400) i |= UF_INVISIBLE;
      if (usr.class & 0x800) i |= UF_NOPASSCH;
      if ((usr.class & 0x300) == 0x100) i = (i & ~UC_MASK) | UC_GROUPOP;
      if ((usr.class & 0x300) == 0x200) i = (i & ~UC_MASK) | UC_SITEOP;
      usr.class = i;
      /**************************************/
      *q = c;
      if (!skipspctocolon(&q)) return -1;
      p = q;
      skiptocolon(&q);
      c  = *q;
      *q = '\0';
      if (!sscanf(p,SI64,&usr.credit)) return -1;
      usr.credit *= 1024;
      *q = c;
      memset(usr.gids,-1,sizeof(usr.gids));
      if (skipspctocolon(&q))
      { i = 0;
	while (i < sizeof(usr.gids) / sizeof(gint32))
	{ skipspace(&q);
	  p = q;
	  while (*q && (*q != ',') && (*q != ':')) q++;
	  if (q > p)
	  { c = *q;
	    *q = '\0';
	    if (!sscanf(p,SI32,usr.gids+i)) return -1;
	    *q = c;
	    if (c) q++;
	    if (++i == sizeof(usr.gids) / sizeof(gint32)) return -1;
	  } else i = sizeof(usr.gids) / sizeof(gint32);
	}
      }
      if (skipspace(&q)) return -1;
    } else inituser(&usr);
  } else
  { strlcpy(usr.info, "", sizeof(usr.info));
    inituser(&usr);
  }
  /* new user data */
  usr.ul_speed = usr.dl_speed = 0;
  usr.dl_week = usr.dl_month = usr.dl_year = 0;
  usr.ul_week = usr.ul_month = usr.ul_year = 0;
  usr.dl_tm_week = usr.dl_tm_month = usr.dl_tm_year = usr.dl_tm_total = 0;
  usr.ul_tm_week = usr.ul_tm_month = usr.ul_tm_year = usr.ul_tm_total = 0;
  /* end of new user data */
  if (!insert_user(&usr))
  { errlog("get_user(): %s",strerror(errno));
    return -1;
  }
  return 0;
}

#ifdef DES_ENCRYPTION
int load_user_file_des_v223(char* filename, int force)
{ char*          buffer;
  C_Block        c;
  FILE*          file;
  long           n;
  C_Block        out;
  Key_schedule   s;

  file = fopen(filename,"rb");
  if (file == NULL)
  { if (!force) return 0;
    errlog("cannot open account file");
    return -1;
  }
  fseek(file,0,SEEK_END);
  n = ftell(file) / sizeof(struct user_s);
  rewind(file);
  if (user_count+n >= user_size)
  { user_size = user_count+n+16;
    user = (struct user_s*)g_realloc(user,user_size*sizeof(struct user_s));
    if (!user)
    { errlog("load_user_file_des(): %s",strerror(errno));
      fclose(file);
      return -1;
    }
  }
  c = user_key;
  des_set_key(&c,&s);
  if (fread(user+user_count,sizeof(struct user_s),n,file) != n)
  { errlog("fread(): %s (%i)",strerror(errno),errno);
    fclose(file);
    return -1;
  }
  fclose(file);
  buffer = (char*)(user+user_count);
  user_count += n;
  n *= sizeof(struct user_s);
  while (n >= 8)
  { des_cbc_encrypt((C_Block*)buffer,&out,8,&s,&c,DES_DECRYPT);
    memmove(buffer,&out,8);
    buffer += 8;
    n -= 8;
  }
  user_dirty = 0;
  return 0;
}
#endif

int load_user_file_v223(char* filename, int force)
{ FILE* file;
  char  line[512];
  int   lineno = 0;
  char* p;

  mk_filename(line,filename,DFL_USERFILE);
#ifdef DES_ENCRYPTION
  if (flag_get(flags,FLAG_DESENCRYPTION)) return load_user_file_des_v223(line,force);
#endif
  file = fopen(line,"rt");
  if (!file)
  { if (!force) return 0;
    errlog("cannot open account file");
    return -1;
  }
  while (readline(line,sizeof(line),file))
  { lineno++;
    p = line;
    skipspace(&p);
    if (*p && (*p!='#')) if (get_user(p) == -1)
    { errlog("load_user_file_v223(): syntax error on line %i (%s)",lineno,line);
      fclose(file);
      return -1;
    }
  }
  fclose(file);
  user_dirty = 0;
  return 0;
}

struct uf_ident_s
{ char id[32];
  int  version;
  int  count;
  int  encrypted;
  int  week;
  int  month;
  int  year;
  int  day; /* This was temp1 changed now to day */
  int  temp2;
} u_id = { "OpenFTPD user database", 101, 0, 0, 0, 0, 0, 0, 0 }; 
/* Version set to 100 to avoid future versions of official FTP4ALL */

int load_user_file_v3012(char* filename, int force)
{  FILE*                file;
   char                 fn_tmp[256];
   struct uf_ident_s    id_test;
   long                 n;
   struct user_v3012_s* old_user = NULL;
   struct user_v3012_s* ou = NULL;
   struct user_s*       u;
   gint32                i;
#ifdef DES_ENCRYPTION
   char*                buffer;
   C_Block              c;
   C_Block              out;
   Key_schedule         s;
#endif   

   mk_filename(fn_tmp,filename,DFL_USERFILE);
   file = fopen(fn_tmp,"rb");
   if (!file) {
      if (!force) return 0;
      errlog("fopen(%s): %s (%i)",fn_tmp,strerror(errno),errno);
      return -1;
   }    
   if (!fread(&id_test,sizeof(id_test),1,file)) id_test.id[0] = 0;
   n = id_test.count;
   if (n > user_size) {
      old_user = (struct user_v3012_s*) g_realloc(NULL, n*sizeof(struct user_v3012_s));
      if (!old_user) {
         errlog("out of memory (user file)");
         fclose(file);
         return -1;
      }
      user_size = n;
   }
   if (fread(old_user,sizeof(struct user_v3012_s),n,file) != n) {
      errlog("fread(user file(v3.012)): %s (%i)",strerror(errno),errno);
      user_count = 0;
      fclose(file);
      return -1;
   }
   user_count = n;
   fclose(file);
   if (id_test.encrypted) {
 #ifdef DES_ENCRYPTION
      c = user_key;
      des_set_key(&c,&s);
      buffer = (char*)old_user;
      n = user_count * sizeof(struct user_v3012_s);
      while (n >= 8) {
         des_cbc_encrypt((C_Block*)buffer,&out,8,&s,&c,DES_DECRYPT);
         memmove(buffer,&out,8);
         buffer += 8;
         n -= 8;
      }
 #else 
      user_count = 0;
      errlog("cannot read crypted user file");
      return -1;
 #endif
   }    
   user = (struct user_s*)g_realloc(user,user_count*sizeof(struct user_s));
   memset(user,0,user_count*sizeof(struct user_s));
   u = user;
   ou = old_user;
   for (i=0; i<user_count; i++) {
      memcpy(u++,old_user++,sizeof(struct user_v3012_s));
   }
   g_free(ou);
   user_dirty = 1;
   return 0;
}

int load_user_file_beta29(char* filename, int force)
{  FILE*                file;
   char                 fn_tmp[256];
   struct uf_ident_s    id_test;
   long                 n;
   struct user_beta29_s* old_user = NULL;
   struct user_beta29_s* ou = NULL;
   struct user_s*       u = NULL;
   gint32                i;
#ifdef DES_ENCRYPTION
   char*                buffer;
   C_Block              c;
   C_Block              out;
   Key_schedule         s;
#endif   

   mk_filename(fn_tmp,filename,DFL_USERFILE);
   file = fopen(fn_tmp,"rb");
   if (!file) {
      if (!force) return 0;
      errlog("fopen(%s): %s (%i)",fn_tmp,strerror(errno),errno);
      return -1;
   }    
   if (!fread(&id_test,sizeof(id_test),1,file)) id_test.id[0] = 0;
   n = id_test.count;
   if (n > user_size) {
      old_user = (struct user_beta29_s*) g_realloc(NULL, n*sizeof(struct user_beta29_s));
      if (!old_user) {
         errlog("out of memory (user file)");
         fclose(file);
         return -1;
      }
      user_size = n;
   }
   if (fread(old_user,sizeof(struct user_beta29_s),n,file) != n) {
      errlog("fread(user file(beta 29)): %s (%i)",strerror(errno),errno);
      user_count = 0;
      fclose(file);
      return -1;
   }
   user_count = n;
   fclose(file);
   if (id_test.encrypted) {
 #ifdef DES_ENCRYPTION
      c = user_key;
      des_set_key(&c,&s);
      buffer = (char*)old_user;
      n = user_count * sizeof(struct user_beta29_s);
      while (n >= 8) {
         des_cbc_encrypt((C_Block*)buffer,&out,8,&s,&c,DES_DECRYPT);
         memmove(buffer,&out,8);
         buffer += 8;
         n -= 8;
      }
 #else 
      user_count = 0;
      errlog("cannot read crypted user file");
      return -1;
 #endif
   }    
   user = (struct user_s*)g_realloc(user,user_count*sizeof(struct user_s));
   memset(user,0,user_count*sizeof(struct user_s));
   u = user;
   ou = old_user;
   for (i=0; i<user_count; i++) {
      memcpy(u++,old_user++,sizeof(struct user_beta29_s));
   }
   g_free(ou);
   user_dirty = 1;
   return 0;
}

int load_user_file(char* filename, int force)
{ FILE*             file;
  char              fn_tmp[256];
  struct uf_ident_s id_test;
  int               n;
  int               new_size;
  struct user_s*    new_user;
#ifdef DES_ENCRYPTION
  char*             buffer;
  C_Block           c;
  C_Block           out;
  Key_schedule      s;
#endif

  mk_filename(fn_tmp,filename,DFL_USERFILE);
  file = fopen(fn_tmp,"rb");
  if (!file)
  { if (!force) return 0;
    errlog("fopen(%s): %s (%i)",fn_tmp,strerror(errno),errno);
    return -1;
  }
  if (!fread(&id_test,sizeof(id_test),1,file))
  { errlog("fread(user file): %s (%i)",strerror(errno),errno);
    return -1;
  }
  if (strcmp(id_test.id,u_id.id) && strcmp(id_test.id, "FTP4ALL user database"))
  { errlog("user file has bad ID, assuming pre 2.23 user file");
    fclose(file);
    return load_user_file_v223(filename,force);
  }
  if (id_test.version > u_id.version)
  { errlog("user file is newer version than daemon");
    fclose(file);
    return -1;
  }
  if (id_test.version < u_id.version)
  { errlog("user file is older version than daemon");
    fclose(file);
    if (id_test.version == 0) {
       errlog("user files is 3.012 version");
       return load_user_file_v3012(filename,force);
    }
    else if (id_test.version == 100) {
       errlog("user files is OpenFTPD beta 29 version");
       return load_user_file_beta29(filename, force);
    }
    return -1;
  }
  n = id_test.count;
  if (n >= user_size)
  { new_size = n + 16;
    new_user = (struct user_s*)g_realloc(user,new_size*sizeof(struct user_s));
    if (!new_user)
    { errlog("out of memory (user file)");
      fclose(file);
      return -1;
    }
    user_size = new_size;
    user = new_user;
  }
  stat_week = id_test.day;
  stat_week = id_test.week;
  stat_month = id_test.month;
  stat_year = id_test.year;
  user_count = fread(user,sizeof(struct user_s),n,file);
  if (user_count != n)
  { errlog("fread(user file): %s (%i)",strerror(errno),errno);
    fclose(file);
    user_count = 0;
    return -1;
  }
  user_count = n;
  fclose(file);
  if (id_test.encrypted)
  {
#ifdef DES_ENCRYPTION
    c = user_key;
    des_set_key(&c,&s);
    buffer = (char*)user;
    n = user_count * sizeof(struct user_s);
    while (n >= 8)
    { des_cbc_encrypt((C_Block*)buffer,&out,8,&s,&c,DES_DECRYPT);
      memmove(buffer,&out,8);
      buffer += 8;
      n -= 8;
    }
#else
    user_count = 0;
    errlog("cannot read crypted user file");
    return -1;
#endif
  }
  user_dirty = 0;
  return 0;
}

int save_user_file(char* filename)
{ FILE*        file;
  char         fn_tmp[256];
  char         fn_tmp2[256];
  size_t       result;
#ifdef DES_ENCRYPTION
  char*        buffer;
  C_Block      c;
  long         n;
  C_Block      out;
  Key_schedule s;
#endif

  mk_filename(fn_tmp,filename,DFL_USERFILE);
  strlcpy(fn_tmp2, fn_tmp, sizeof(fn_tmp2));
  strlcat(fn_tmp, ".tmp", sizeof(fn_tmp));
  file = fopen(fn_tmp,"wb");
  if (file == NULL)
  { errlog("fopen(%s): %s (%i)",fn_tmp,strerror(errno),errno);
    return -1;
  }
  u_id.count = user_count;
#ifdef DES_ENCRYPTION
  u_id.encrypted = flag_get(flags,FLAG_DESENCRYPTION);
#endif
  u_id.day = stat_day;
  u_id.week = stat_week;
  u_id.month = stat_month;
  u_id.year = stat_year;
  if (!fwrite(&u_id,sizeof(u_id),1,file))
  { errlog("fwrite(%s): %s (%i)",fn_tmp,strerror(errno),errno);
    fclose(file);
    return -1;
  }
#ifdef DES_ENCRYPTION
  if (u_id.encrypted)
  { c = user_key;
    des_set_key(&c,&s);
    buffer = (char*)user;
    n = user_count * sizeof(struct user_s);
    while (n >= 8)
    { des_cbc_encrypt((C_Block*)buffer,&out,8,&s,&c,DES_ENCRYPT);
      memmove(buffer,&out,8);
      buffer += 8;
      n -= 8;
    }
  }
#endif
  result = fwrite(user,sizeof(struct user_s),user_count,file);
  if (result != user_count)
  { errlog("fwrite(%s): %s (%i)",fn_tmp,strerror(errno),errno);
    fclose(file);
  } else
  { fclose(file);
    if (rename(fn_tmp,fn_tmp2) < 0)
      errlog("rename(%s,%s): %s (%i)",fn_tmp,fn_tmp2,strerror(errno),errno);
  }
#ifdef DES_ENCRYPTION
  if (u_id.encrypted)
  { c = user_key;
    des_set_key(&c,&s);
    buffer = (char*)user;
    n = user_count * sizeof(struct user_s);
    while (n >= 8)
    { des_cbc_encrypt((C_Block*)buffer,&out,8,&s,&c,DES_DECRYPT);
      memmove(buffer,&out,8);
      buffer += 8;
      n -= 8;
    }
  }
#endif
  return result == user_count ? 0 : -1;
}

void update_updown(struct child_s* c, struct user_s* u, gint64 up, 
                   gint64 down, gint64 uptime, gint64 downtime, gint32 day,
                   gint32 week, gint32 month, gint32 year, gint32 total,
                   struct section_s *s) 
{ 
   struct sstat_s *st;
   int no_stats;

   st = get_stat_by_uid(s, (int)u->uid);
   if(!st) return;

   no_stats = get_key_value_by_section(s, "no_global_stats", NULL, 0, NULL);
   if (!up && !down) return;
   user_dirty++;
   section_stat_dirty++;
   if (up > 0) { 
      if (!no_stats) {
         u->ul_f_day += day;
         u->ul_f_week += week;
         u->ul_f_month +=month;
         u->ul_f_year +=year;
         u->ul_f_total +=total;
      }
      st->ul_f_day +=day;
      st->ul_f_week +=week;
      st->ul_f_month +=month;
      st->ul_f_year +=year;
      st->ul_f_total +=total;
      if (uptime) { 
         if (!no_stats) {
            if (day) {
               if (u->ul_day >= 0) {
                  u->ul_tm_day += uptime; 
               }
               else if (u->ul_day+up > 0) {
                  u->ul_tm_day = (uptime*(u->ul_day+up))/up;
               }
            }
            if (week) {
               if (u->ul_week >= 0) {
   	          u->ul_tm_week += uptime; 
	       }
	       else if (u->ul_week+up > 0) {
	          u->ul_tm_week = (uptime*(u->ul_week+up))/up;
               }
            }
            if (month) {
	       if (u->ul_month >= 0) {
	          u->ul_tm_month += uptime; 
	       }
	       else if (u->ul_month+up > 0) {
	          u->ul_tm_month = (uptime*(u->ul_month+up))/up;
               }
            }
            if (year) {
               if (u->ul_year >= 0) { 
	          u->ul_tm_year += uptime; 
	       }   
	       else if (u->ul_year+up > 0) {
                  u->ul_tm_year = (uptime*(u->ul_year+up))/up;
               }
            }
            if (total) {
               if (u->ul_total >= 0) {
	          u->ul_tm_total += uptime; 
	       }   
	       else if (u->ul_total+up > 0) {
                  u->ul_tm_total = (uptime*(u->ul_total+up))/up;
               }
            }
         }
         if (day) {
            if (st->ul_day >= 0) {
 	       st->ul_tm_day += uptime; 
	    }
	    else if (st->ul_day+up > 0) {
	       st->ul_tm_day = (uptime*(st->ul_day+up))/up;
            }
         }
         if (week) {
            if (st->ul_week >= 0) {
	       st->ul_tm_week += uptime; 
	    }
	    else if (st->ul_week+up > 0) {
	       st->ul_tm_week = (uptime*(st->ul_week+up))/up;
            }
         }
         if (month) {
	    if (st->ul_month >= 0) {
	       st->ul_tm_month += uptime; 
	    }
	    else if (st->ul_month+up > 0) {
	       st->ul_tm_month = (uptime*(st->ul_month+up))/up;
            }
         }
         if (year) {
            if (st->ul_year >= 0) { 
	       st->ul_tm_year += uptime; 
	    }   
	    else if (st->ul_year+up > 0) {
               st->ul_tm_year = (uptime*(st->ul_year+up))/up;
            }
         }
         if (total) {
            if (st->ul_total >= 0) {
	       st->ul_tm_total += uptime; 
	    }   
	    else if (st->ul_total+up > 0) {
               st->ul_tm_total = (uptime*(st->ul_total+up))/up;
            }
         }

         if (c) { 
	    if (c->ul >= 0) c->ul_tm += uptime; 
	    else if (c->ul+up > 0) c->ul_tm = (uptime*(c->ul+up))/up; 
	 }
      } 
      else {
         if (!no_stats) { 
           if (day) 
            if (u->ul_day > 0) u->ul_tm_day += (up*u->ul_tm_day)/u->ul_day;
           if (week)
            if (u->ul_week > 0) u->ul_tm_week += (up*u->ul_tm_week)/u->ul_week;
           if (month)
            if (u->ul_month > 0) u->ul_tm_month += (up*u->ul_tm_month)/u->ul_month;
           if (year)
            if (u->ul_year > 0) u->ul_tm_year += (up*u->ul_tm_year)/u->ul_year;
           if (total)
            if (u->ul_total > 0) u->ul_tm_total += (up*u->ul_tm_total)/u->ul_total;
         }
         if (day)
         if (st->ul_day > 0) st->ul_tm_day += (up*st->ul_tm_day)/st->ul_day;
         if (week)
         if (st->ul_week > 0) st->ul_tm_week += (up*st->ul_tm_week)/st->ul_week;
         if (month)
         if (st->ul_month > 0) st->ul_tm_month += (up*st->ul_tm_month)/st->ul_month;
         if (year)
         if (st->ul_year > 0) st->ul_tm_year += (up*st->ul_tm_year)/st->ul_year;
         if (total)
         if (st->ul_total > 0) st->ul_tm_total += (up*st->ul_tm_total)/st->ul_total;

         if (c) if (c->ul > 0) u->ul_tm_total += (up*c->ul_tm)/c->ul;
      }
   } 
   else if (up < 0) { 
      if (!no_stats) {
         u->ul_f_day -= day;
         u->ul_f_week -= week;
         u->ul_f_month -= month;
         u->ul_f_year -= year;
         u->ul_f_total -= total;
    
         if (day) {
            if (u->ul_day >= 0) { 
               if (u->ul_day+up <= 0) u->ul_tm_day = 0; 
               else u->ul_tm_day += (up*u->ul_tm_day)/u->ul_day; 
            }
         }
         if (week) {
            if (u->ul_week >= 0) { 
               if (u->ul_week+up <= 0) u->ul_tm_week = 0; 
               else u->ul_tm_week += (up*u->ul_tm_week)/u->ul_week; 
            }
         }
         if (month) {
            if (u->ul_month >= 0) { 
               if (u->ul_month+up <= 0) u->ul_tm_month = 0; 
	       else u->ul_tm_month += (up*u->ul_tm_month)/u->ul_month; 
            }
         }
         if (year) {
            if (u->ul_year >= 0) { 
               if (u->ul_year+up <= 0) u->ul_tm_year = 0; 
               else u->ul_tm_year += (up*u->ul_tm_year)/u->ul_year; 
            }
         }
         if (total) {
            if (u->ul_total >= 0) { 
               if (u->ul_total+up <= 0) u->ul_tm_total = 0; 
	       else u->ul_tm_total += (up*u->ul_tm_total)/u->ul_total; 
            }
         }
      }
      if (day) {
         st->ul_f_day -= day;
         if (st->ul_day >= 0) { 
            if (st->ul_day+up <= 0) st->ul_tm_day = 0; 
	    else st->ul_tm_day += (up*st->ul_tm_day)/st->ul_day; 
         }
      }
      if (week) {
         st->ul_f_week -= week;
         if (st->ul_week >= 0) { 
            if (st->ul_week+up <= 0) st->ul_tm_week = 0; 
            else st->ul_tm_week += (up*st->ul_tm_week)/st->ul_week; 
         }
      }
      if (month) {
         st->ul_f_month -= month;
         if (st->ul_month >= 0) { 
            if (st->ul_month+up <= 0) st->ul_tm_month = 0; 
	    else st->ul_tm_month += (up*st->ul_tm_month)/st->ul_month; 
         }
      }
      if (year) {
         st->ul_f_year -= year;
         if (st->ul_year >= 0) { 
            if (st->ul_year+up <= 0) st->ul_tm_year = 0; 
            else st->ul_tm_year += (up*st->ul_tm_year)/st->ul_year; 
         }
      }
      if (total) {
         st->ul_f_total -= total;
         if (st->ul_total >= 0) { 
            if (st->ul_total+up <= 0) st->ul_tm_total = 0; 
	    else st->ul_tm_total += (up*st->ul_tm_total)/st->ul_total; 
         }
      }

      if (c) {
         if (c->ul >= 0) { 
            if (c->ul+up <= 0) c->ul_tm = 0; 
	    else c->ul_tm += (up*c->ul_tm)/c->ul; 
         }
      }	 
   }
   if (down > 0) { 
      if (!no_stats) {
         u->dl_f_day += day;
         u->dl_f_week += week;
         u->dl_f_month += month;
         u->dl_f_year += year;
         u->dl_f_total += total;
      }
      st->dl_f_day += day;
      st->dl_f_week += week;
      st->dl_f_month += month;
      st->dl_f_year += year;
      st->dl_f_total += total;
      if (downtime) { 
         if (!no_stats) {
            if (day) {
               if (u->dl_day >= 0) {
                  u->dl_tm_day += downtime; 
	       }
	       else if (u->dl_day+down > 0) {
                  u->dl_tm_day = (downtime*(u->dl_day+down))/down;
               }
            }
            if (week) {
               if (u->dl_week >= 0) {
	          u->dl_tm_week += downtime; 
	       }
	       else if (u->dl_week+down > 0) {
                  u->dl_tm_week = (downtime*(u->dl_week+down))/down;
               }
            }
            if (month) {
               if (u->dl_month >= 0) {
	          u->dl_tm_month += downtime; 
	       }   
	       else if (u->dl_month+down > 0) {
                  u->dl_tm_month = (downtime*(u->dl_month+down))/down;
               }
            }
            if (year) {
               if (u->dl_year >= 0) {
	          u->dl_tm_year += downtime; 
	       }   
	       else if (u->dl_year+down > 0) {
                  u->dl_tm_year = (downtime*(u->dl_year+down))/down;
               }
            }
            if (total) {
               if (u->dl_total >= 0) {
	          u->dl_tm_total += downtime; 
	       }   
	       else if (u->dl_total+down > 0) {
                  u->dl_tm_total = (downtime*(u->dl_total+down))/down;
               }
            }
         }
         if (day) {
            if (st->dl_day >= 0) {
	       st->dl_tm_day += downtime; 
	    }
	    else if (st->dl_day+down > 0) {
               st->dl_tm_day = (downtime*(st->dl_day+down))/down;
            }  
         }
         if (week) {
            if (st->dl_week >= 0) {
	       st->dl_tm_week += downtime; 
	    }
	    else if (st->dl_week+down > 0) {
               st->dl_tm_week = (downtime*(st->dl_week+down))/down;
            }
         }
         if (month) {
            if (st->dl_month >= 0) {
	       st->dl_tm_month += downtime; 
	    }   
	    else if (st->dl_month+down > 0) {
               st->dl_tm_month = (downtime*(st->dl_month+down))/down;
            }
         }
         if (year) {
            if (st->dl_year >= 0) {
	       st->dl_tm_year += downtime; 
	    }   
	    else if (st->dl_year+down > 0) {
               st->dl_tm_year = (downtime*(st->dl_year+down))/down;
            }
         }
         if (total) {
            if (st->dl_total >= 0) {
	       st->dl_tm_total += downtime; 
	    }   
	    else if (st->dl_total+down > 0) {
               st->dl_tm_total = (downtime*(st->dl_total+down))/down;
            }
         }

         if (c) { 
	    if (c->dl >= 0) c->dl_tm += downtime; 
	    else if (c->dl+down > 0) c->dl_tm = (downtime*(c->dl+down))/down; 
	 }
      } 
      else { 
         if (!no_stats) {
            if (day)
            if (u->dl_day > 0) u->dl_tm_day += (down*u->dl_tm_day)/u->dl_day;
            if (week)
            if (u->dl_week > 0) u->dl_tm_week += (down*u->dl_tm_week)/u->dl_week;
            if (month)
            if (u->dl_month > 0) u->dl_tm_month += (down*u->dl_tm_month)/u->dl_month;
            if (year)
            if (u->dl_year > 0) u->dl_tm_year += (down*u->dl_tm_year)/u->dl_year;
            if (total)
            if (u->dl_total > 0) u->dl_tm_total += (down*u->dl_tm_total)/u->dl_total;
         }
         if (day)
         if (st->dl_day > 0) st->dl_tm_day += (down*st->dl_tm_day)/st->dl_day;
         if (week)
         if (st->dl_week > 0) st->dl_tm_week += (down*st->dl_tm_week)/st->dl_week;
         if (month)
         if (st->dl_month > 0) st->dl_tm_month += (down*st->dl_tm_month)/st->dl_month;
         if (year)
         if (st->dl_year > 0) st->dl_tm_year += (down*st->dl_tm_year)/st->dl_year;
         if (total)
         if (st->dl_total > 0) st->dl_tm_total += (down*st->dl_tm_total)/st->dl_total;

         if (c) if (c->dl > 0) c->dl_tm += (down*c->dl_tm)/c->dl;
      }
   } 
   else if (down < 0) { 
      if (no_stats) {
         u->dl_f_day -= day;
         u->dl_f_week -= week;
         u->dl_f_month -= month;
         u->dl_f_year -= year;
         u->dl_f_total -= total;
         if (day) {
            if (u->dl_day >= 0) { 
               if (u->dl_day+down <= 0) u->dl_tm_day = 0; 
               else u->dl_tm_day += (down*u->dl_tm_day)/u->dl_day; 
            }
         }
         if (week) {
            if (u->dl_week >= 0) { 
               if (u->dl_week+down <= 0) u->dl_tm_week = 0; 
	       else u->dl_tm_week += (down*u->dl_tm_week)/u->dl_week; 
            }
         }
         if (month) {
            if (u->dl_month >= 0) { 
               if (u->dl_month+down <= 0) u->dl_tm_month = 0; 
	       else u->dl_tm_month += (down*u->dl_tm_month)/u->dl_month; 
            }
         }
         if (year) { 
            if (u->dl_year >= 0) { 
               if (u->dl_year+down <= 0) u->dl_tm_year = 0; 
	       else u->dl_tm_year += (down*u->dl_tm_year)/u->dl_year; 
            }
         }
         if (total) {
            if (u->dl_total >= 0) { 
               if (u->dl_total+down <= 0) u->dl_tm_total = 0; 
	       else u->dl_tm_total += (down*u->dl_tm_total)/u->dl_total; 
            }
         }
      }
      st->dl_f_day -= day;
      st->dl_f_week -= week;
      st->dl_f_month -= month;
      st->dl_f_year -= year;
      st->dl_f_total -= total;
      if (day) {
         if (st->dl_day >= 0) { 
            if (st->dl_day+down <= 0) st->dl_tm_day = 0; 
	    else st->dl_tm_day += (down*st->dl_tm_day)/st->dl_day; 
         }
      }
      if (week) {
         if (st->dl_week >= 0) { 
            if (st->dl_week+down <= 0) st->dl_tm_week = 0; 
	    else st->dl_tm_week += (down*st->dl_tm_week)/st->dl_week; 
         }
      }
      if (month) {
         if (st->dl_month >= 0) { 
            if (st->dl_month+down <= 0) st->dl_tm_month = 0; 
	    else st->dl_tm_month += (down*st->dl_tm_month)/st->dl_month; 
         }
      }
      if (year) {
         if (st->dl_year >= 0) { 
            if (st->dl_year+down <= 0) st->dl_tm_year = 0; 
  	    else st->dl_tm_year += (down*st->dl_tm_year)/st->dl_year; 
         }
      }
      if (total) {
         if (st->dl_total >= 0) { 
            if (st->dl_total+down <= 0) st->dl_tm_total = 0; 
            else st->dl_tm_total += (down*st->dl_tm_total)/st->dl_total; 
         }
      }

      if (c) {
         if (c->dl >= 0) { 
	    if (c->dl+down <= 0) c->dl_tm = 0; 
	    else c->dl_tm += (down*c->dl_tm)/c->dl; 
	 }
      }	
   }
   if (day) {
      st->dl_day  += down;
      st->ul_day  += up;
      if (!no_stats) {
         u->dl_day  += down;
         u->ul_day  += up;
      }
   }
   if (week) {
      st->dl_week  += down;
      st->ul_week  += up;
      if (!no_stats) {
         u->dl_week  += down;
         u->ul_week  += up;
      }
   }
   if (month) {
      st->dl_month += down;
      st->ul_month += up;
      if (!no_stats) {
         u->dl_month += down;
         u->ul_month += up;
      }
   }
   if (year) {
      st->dl_year  += down;
      st->ul_year  += up;
      if (!no_stats) {
         u->dl_year  += down;
         u->ul_year  += up;
      }
   }
   if (total) {
      st->ul_total += up;
      st->dl_total += down;
      if (!no_stats) {
         u->dl_total += down;
         u->ul_total += up;
      }
   }

   if (c) { 
      c->dl += down;
      c->ul += up;
   }
}

int account_disabled(struct user_s* u)
{ return u ? (u->class & UF_DISABLED) : 0;
}

int anonuser(struct user_s* u)
{ return u ? ((u->class & UC_MASK) == UC_ANON) : 0;
}

int coloruser(struct user_s* u)
{ return u ? (u->class & UF_COLOR) : 0;
}

int groupop(struct user_s* u)
{ return u ? ((u->class & UC_MASK) == UC_GROUPOP) : 0;
}

int groupopof(struct user_s* u, gint32 gid)
{ int i;

  if (gid == -1) return 0;
  if (u->gid == gid) return 1;
  for (i=0; i<8; i++) if (u->gids[i] == gid) return 1;
  return 0;
}

int invisible(struct user_s* u)
{ return u ? (u->class & UF_INVISIBLE) : 0;
}

int no_pass_change(struct user_s* u)
{ return u ? (u->class & UF_NOPASSCH) : 0;
}

int no_credit(struct user_s* u)
{ return u ? (u->class & UF_NOCREDIT) : 0;
}

int normaluser(struct user_s* u)
{ return u ? ((u->class & UC_MASK) == UC_NORMAL) : 0;
}

int siteop(struct user_s* u)
{ return u ? ((u->class & UC_MASK) == UC_SITEOP) : 0;
}

int superuser(struct user_s* u)
{ return u ? ((u->class & UC_MASK) == UC_ROOT) : 0;
}

int templateuser(struct user_s* u)
{ return u ? ((u->class & UC_MASK) == UC_TEMPLATE) : 0;
}

int allowed(struct user_s* u, gint64 right, struct user_s* target)
{ 
   int   i;
   gint64 r;

   /* noone can delete himself, not even superuser */
   if ((u == target) && (right == R_USER_DELETE)) return 0;
   /* superuser can do anything */
   if (superuser(u)) return 1;

   r = right;
   if (target) {
      if (r == R_USER_READ && u == target ) return 1;
      if (groupop(target) && r == R_USER_READ) r = R_USER_READ_GO;
      if (siteop(target)  && r == R_USER_READ) r = R_USER_READ_SO;
      if (superuser(target) && r == R_USER_READ) r = R_USER_READ_SO;
   }

   switch (r) { 
      case R_USER_READ:
      case R_USER_READ_SO:
      case R_USER_READ_GO:
      case R_GROUP_LIST:
      case R_GROUP_ALL:
      case R_USER_ALL:
      case R_ALWAYS_LOGIN: 
      case R_ALL: return (u->rights & r) != 0 ; break;
      default: r = 0;
   } 
   /* siteop checks */
   if (siteop(u)) { 
      if (target) { 
         if (superuser(target)) return 0;
         if (u == target) return 1; /* siteops can edit themself */
	 if (siteop(target)) { 
	    switch (right) { 
	       case R_USER_DELETE : r = R_USER_DELETE_SO; break;
               case R_USER_EDIT   : r = R_USER_EDIT_SO;   break;
               case R_USER_READ   : r = (u == target) ? u->rights : R_USER_READ_SO;
                        	    break;
               default            : r = 0;
            }
            if (!(u->rights & r)) return 0;
         }
/* Fixes the "Siteop can't change groupops infomation bug"
	 if (groupop(target)) { 
            switch (right) { 
	       case R_USER_DELETE  : r = R_USER_DELETE_GO; break;
               case R_USER_EDIT    : r = R_USER_EDIT_GO;   break;
               case R_USER_READ    : r = R_USER_READ_GO;   break;
               case R_USER_DISABLE :
               case R_USER_KICK    :
               case R_USER_PASSWD  : r = right;            break;
               default             : r = 0;
            } 
            if (!(u->rights & r)) return 0;
	 } */
         if ((u == target) && (right == R_USER_EDIT_RIGHTS)) return 0;
      }
      return (u->rights & right) != 0;
   }

   /* groupop checks */
   if (groupop(u))
   { if (target)
     { if (superuser(target) || siteop(target)) return 0;
       if (groupop(target))
       { switch (right)
         { case R_USER_DELETE : r = R_USER_DELETE_GO; break;
           case R_USER_EDIT   : r = R_USER_EDIT_GO;   break;
           case R_USER_READ   : r = (u == target) ? u->rights : R_USER_READ_GO;
           case R_USER_EDIT_IP: r = (u == target || u->gid == target->gid) ? 
                                     u->rights : 0;
                        	break;
           default            : r = 0;
         }
         if (!(u->rights & r)) return 0;
       }
       if ((u == target) && (right == R_USER_EDIT_RIGHTS)) return 0;
       if (target->gid == -1) return 0;
       if (u->gid == target->gid) return (u->rights & right) != 0;
       for (i=0; i<8; i++) if (u->gids[i] == target->gid)
         return (u->rights & right) != 0;
       return 0;
     }
     return (u->rights & right) != 0;
   }
   return 0;
}

struct user_s* get_user_by_uid(gint32 uid)
{ int            i = 0;
  int            j;
  int            k = user_count - 1;

  while (i <= k)
  { j = (i + k) >> 1;
    if (user[j].uid < uid) i = j + 1; else if (user[j].uid > uid) k = j - 1;
      else
    { while (j && (user[j-1].uid == uid)) j--;
      return user+j;
    }
  }
  return NULL;
}

struct user_s* get_user_by_name(char* name)
{ int            i;
  struct user_s* u;
  gint32          uid;

  for (i=0, u=user; i<user_count; i++, u++) if (!strcasecmp(u->name,name))
    return u;
  if (sscanf(name,SI32,&uid)) return get_user_by_uid(uid);
  return NULL;
}

void print_iprange_part(char* s, char l, char h, char c)
{ if (l == h) sprintf(s,"%i",(int)(unsigned char)l); else
  if ((l == 0) && ((unsigned char)h == 0xFF)) sprintf(s,"*"); else
    sprintf(s,"%i-%i",(int)(unsigned char)l,(int)(unsigned char)h);
  sprintf(s+strlen(s),"%c",c);
}

void print_iprange(char* s, char* iprange)
{ print_iprange_part(s,*(iprange+12),*(iprange+13),'.');
  print_iprange_part(s+strlen(s),*(iprange+14),*(iprange+15),'.');
  print_iprange_part(s+strlen(s),*(iprange+16),*(iprange+17),'.');
  print_iprange_part(s+strlen(s),*(iprange+18),*(iprange+19),'\0');
}

void print_ident(char* s, char* ident)
{ if (*ident) sprintf(s,"%s@",ident); else *s = '\0';
  print_iprange(s+strlen(s),ident);
}

void create_initial_superuser(void)
{ struct user_s u;

  memset(&u,0,sizeof(u));
  memset(u.gids,-1,sizeof(u.gids));
  strlcpy(u.name, "root", sizeof(u.name));
  u.class = UC_ROOT;
  u.idents[0][13] = u.idents[0][15] = u.idents[0][17] = u.idents[0][19] = 255;
  if (insert_user(&u))
    errlog("initial superuser created"); else
    errlog("creation of initial superuser failed");
}

void restore_initial_superuser(void)
{ struct user_s* u;

  u = get_user_by_name("root");
  if (u)
  { u->password[0] = 0;
    u->class &= ~UC_MASK;
    u->class |= UC_ROOT;
    u->class &= ~UF_DISABLED;
    memset(u->idents[0],0,sizeof(u->idents[0]));
    u->idents[0][13] = u->idents[0][15] = u->idents[0][17] =
      u->idents[0][19] = 255;
    errlog("initial superuser restored");
    user_dirty++;
  } else create_initial_superuser();
}

gint32 get_new_uid(void)
{ return user_count ? user[user_count-1].uid + 1 : 0;
}

void add_weekly_credits(void)
{  
   int            i, n;
   struct user_s* u;
   struct sstat_s *st;
   struct section_s *s;

   for (i=0, u=user; i < user_count; i++, u++) {
      if (u->wkly_alot != 0 && u->credit < u->wkly_alot)
         u->credit = u->wkly_alot;
   }
   for (n = 0, s = section; n < section_count; n++, s++) {
      if (get_key_value_by_section(s, "different_credit", NULL, 0, NULL)) {
         for (i = 0, st = s->stat; i < s->stat_count; i++, st++) {
            if (st->wkly_alot != 0 && st->credit < st->wkly_alot) 
               st->credit = st->wkly_alot;
         }
      }
   }
}

/* EOF */
