/*   config.c
 * 
 *   This file is part of FTP4ALL
 *
 *   Copyright (C) 1996-1999 by Crescent
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sys/types.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include "externs.h"
#include "common.h"
#include "log.h"
#include "utils.h"
#include "cdpath.h"
#include "strlcpy.h"
#include "strlcat.h"

int load_cdpath(void)
{ 
  gchar buf[512];
  FILE* file;

  mk_filename (buf, "etc/cdpath.cfg", NULL);

  file = fopen(buf,"rt");

  if (!file)
    return -1;
  
  cdpath_reset ();

  while (readline (buf, sizeof (buf), file))
    {
      if (*buf != '#') 
	{
	  gint32 uid = 0, gid = 0, perm = 0;
	  gchar path[1024] = { 0, };

	  if (sscanf (buf, "%1024s " SI32 " " SI32 " " SI32, 
		      path, &uid, &gid, &perm) == 4)
	    cdpath_set (path, uid, gid, perm);
	  else
	    g_printerr ("load_cdpath: bad config line:\n %s\n",	buf);
	}
    }
  
  fclose(file);

  return 0;
}

int get_check(char* p, struct check_s** check, int* check_count, int* check_size)
{ 
   struct check_s* ch;
   char            name[sizeof(ch->name)];
   char            param[sizeof(ch->param)];
   char*           q = p;
   char            wildcard[sizeof(ch->wildcard)];

  if (skipnspace(&q))
  { if (q-p >= sizeof(wildcard)) return -1;
    strncpy(wildcard,p,q-p);
    *(wildcard+(q-p)) = '\0';
    if (!skipspace(&q)) return -1;
    p = q;
  } else return -1;
  skipnspace(&q);
  if (q-p >= sizeof(name)) return -1;
  strncpy(name,p,q-p);
  *(name+(q-p)) = '\0';
  if (*q)
  { if (skipspace(&q))
    { p = q;
      skipnspace(&q);
      if (q-p >= sizeof(param)) return -1;
      strncpy(param,p,q-p);
      *(param+(q-p)) = '\0';
      if (skipspace(&q)) return -1;
    } else *param = '\0';
  } else *param = '\0';
  if (*check_count == *check_size) {
    if(!(ch = (struct check_s*) g_realloc( (char*)*check, (*check_size + 4 * sizeof(char)) * sizeof(struct check_s)))) {
      errlog("get_check(): %s (%i)",strerror(errno),errno);
      return -1;
    }
    *check = ch;
    *check_size += 4 * sizeof(char);
  }
  ch = *check + *check_count;
  (*check_count)++;
  strlcpy(ch->wildcard, wildcard, sizeof(ch->wildcard));
  strlcpy(ch->name, name, sizeof(ch->name));
  strlcpy(ch->param, param, sizeof(ch->param));
  return 0;
}

int load_check(void)
{ char  buf[512];
  FILE* file;

  mk_filename(buf,"etc/check.cfg",NULL);
  file = fopen(buf,"rt");
  if (!file) return -1;
  check_count = 0;
  while (readline(buf,sizeof(buf),file))
  { if (*buf != '#') get_check(buf,&check,&check_count,&check_size);
  }
  fclose(file);
  return 0;
}

int load_dcheck(void)
{ 
  char  buf[512];
  FILE* file;

  mk_filename(buf, "etc/checkdupe.cfg", NULL);
  file = fopen(buf,"rt");
  if (!file) return -1;
  dcheck_count = 0;
  while (readline(buf, sizeof(buf), file)) {
     if (*buf != '#') get_check(buf, &dcheck, &dcheck_count, &dcheck_size);
  }
  fclose(file);
  return 0;
}

int load_dirdcheck(void)
{ 
  char  buf[512];
  FILE* file;

  mk_filename(buf, "etc/checkdirdupe.cfg", NULL);
  file = fopen(buf, "rt");
  if (!file) return -1;
  dirdcheck_count = 0;
  while (readline(buf,sizeof(buf),file)) {
     if (*buf != '#') get_check(buf, &dirdcheck, &dirdcheck_count, &dirdcheck_size);
  }
  fclose(file);
  return 0;
}

int load_delcheck(void)
{
   char  buf[512];
   FILE* file;

   mk_filename(buf, "etc/checkdelete.cfg", NULL);
   file = fopen(buf, "rt");
   if (!file) return -1;
   delcheck_count = 0;
   while (readline(buf, sizeof(buf), file)) {
      if (*buf != '#') get_check(buf, &delcheck, &delcheck_count, &delcheck_size);
   }
   fclose(file);
   return 0;
 }

int get_dshortcut(char* buf) {
  char  *dsh;
  size_t dsh_len; 
  int    l1;
  int    l2;
  int    n;
  char  *p[8];

  n = 0;
  parse_command(buf,p,8,&n);
  if (n != 2) return -1;
  l1 = strlen(p[0]);
  l2 = strlen(p[1]);
  while (dshortcut_len+l1+l2+2 >= dshortcut_size) {
    /* dsh = g_realloc(dshortcut,dshortcut_size+256); */
    dsh_len = dshortcut_size + 256 * sizeof(char);
    if((dsh = g_realloc(dshortcut, dsh_len)) == NULL) {
      errlog("get_dshortcut(): %s (%i)",strerror(errno),errno);
      return -1;
    }
    dshortcut = dsh;
    dshortcut_size += 256;
  }
  dsh = dshortcut + dshortcut_len;
  strlcpy(dsh, p[0], dsh_len);
  dsh += l1 + sizeof(char);
  strcpy(dsh,p[1]);
  dshortcut_len += l1 + l2 + 2 * sizeof(char);
  return 0;
}

int load_dshortcut(void)
{ char  buf[512];
  FILE* file;

  mk_filename(buf,"etc/dirshortcut.cfg",NULL);
  file = fopen(buf,"rt");
  if (!file) return -1;
  dshortcut_len = 0;
  while (readline(buf,sizeof(buf),file))
  { if (*buf != '#') get_dshortcut(buf);
  }
  fclose(file);
  return 0;
}

/* added by primemover */
int get_customcmd(char* p, struct customcmd_s** customcmd, int* customcmd_count, int* customcmd_size)
{ 
   struct customcmd_s* sc;
   char              path[sizeof(sc->path)];
   char              param[sizeof(sc->param)];
   char*             q = p;
   char              command[sizeof(sc->command)];

   if (skipnspace(&q)) { 
      if (q - p >= sizeof(command)) return -1;
      strncpy(command, p, q-p);
      *(command + (q - p)) = '\0';
      if (!skipspace(&q)) return -1;
      p = q;
   } else return -1;

   skipnspace(&q);
   if (q - p >= sizeof(path)) return -1;
   strncpy(path, p, q-p);
   *(path + (q - p)) = '\0';
   if (*q){ 
   if (skipspace(&q)) { 
      if(strlen(q) >= sizeof(param)) return -1;
      strncpy(param, q, sizeof(param) - 1);
      if(*(param+strlen(param)-1) == '\n') 
        *(param+strlen(param)-1) = 0;
    } else *param = '\0';
  } else *param = '\0';
  if (*customcmd_count == *customcmd_size) { 
     sc = (struct customcmd_s*) g_realloc((char*) *customcmd,
          (*customcmd_size + 4) *sizeof(struct customcmd_s));
     if (!sc) { 
        errlog("get_customcmd(): %s (%i)",strerror(errno),errno);
        return -1;
     }
     *customcmd = sc;
     *customcmd_size += 4;
  }
  sc = *customcmd + *customcmd_count;
  (*customcmd_count)++;
  strlcpy(sc->command, command, sizeof(sc->command));
  strlcpy(sc->path, path, sizeof(sc->path));
  strlcpy(sc->param, param, sizeof(sc->param));
  return 0;
}

int load_customcmd(void)
{ 
   char  buf[512];
   FILE* file;

   mk_filename(buf,"etc/customcmd.cfg",NULL);
   file = fopen(buf,"rt");
   if (!file) return -1;
   customcmd_count = 0;
   while (readline(buf, sizeof(buf), file)) { 
      if ((*buf != '#') && (*buf != ';')) get_customcmd(buf, &customcmd, &customcmd_count, &customcmd_size);
   }
   fclose(file);
   return 0;
}

/* EOF */
