/* ---------------------------------------------------------------------------
 *
 *   This file is part of OpenFTPD (based on FTP4ALL)
 *   (C) 1996-2000 by Crescent, (C) 2000 by the OpenFTPD Team
 *
 * ---------------------------------------------------------------------------
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 * ---------------------------------------------------------------------------
 *   MODULE: nuke.c - nuke files or directories
 * ---------------------------------------------------------------------------
 */

#include "ac_config.h"

#include <sys/types.h>
#include <ctype.h>
#include <dirent.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/signal.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include <time.h>
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
#include "externs.h"
#include "common.h"
#include "ftps.h"
#include "list.h"
#include "log.h"
#include "utils.h"
#include "perm.h"
#include "site.h"
#include "site_vfs.h"
#include "strlcpy.h"
#include "strlcat.h"

#define DFL_NUKEPREFIX "_NUKED_"
#define DFL_EMPTYSIZE  25

/* check for nuke permission */
int nuker(char *username) 
{
   char  str[256];

   printf("RGT_CHK %s R_NUKE\n", username);
   if (!fgets(str, sizeof(str),stdin)) strlcpy(str, "-pipe error", sizeof(str));
   if (*str != '+') return 0;
   return 1;
}

      
void print_nuke_head()
{
  char str[256];

  strlcpy(str, " !C.------------------------------------------------------------------------.!0", sizeof(str));
  send_message(211, 0, str, NULL);
  strlcpy(str, " !C|!3 !HNukee        Filesize   Filename                                       !0!C|!0", sizeof(str));
  send_message(211, 0, str, NULL);
  strlcpy(str, " !C|------------------------------------------------------------------------|!0", sizeof(str));
  send_message(211, 0, str, NULL);
}  

void print_nuke_foot(float size, float factor)
{
  char str[256];

  strlcpy(str, " !C|------------------------------------------------------------------------|!0", sizeof(str));
  send_message(211, 0, str, NULL);
  g_snprintf(str, sizeof(str), " !C|!0 !CTOTALSIZE: !H%7.1f MB   !CFACTOR: !H%4.1f                                   !0!C|!0", 
     (float) size / 0x100000, factor);
  send_message(211, 0, str, NULL);
  strlcpy(str, " !C`------------------------------------------------------------------------+!0", sizeof(str));
  send_message(211, 0, str, NULL);
} 


int update_stats(char *username, int id, gint64 size, int times) 
{
   char	 str[256];

   switch (times) {
      case 0: printf("UUU %s "SI64" 1 1 1 1 1 %d\n", 
                 username, size, id);
              break;
      case 1: printf("UUU %s "SI64" 0 1 1 1 1 %d\n", 
                 username, size, id);
              break;	 
      case 2: printf("UUU %s "SI64" 0 0 1 1 1 %d\n", 
                 username, size, id);
              break;	 
      case 3: printf("UUU %s "SI64" 0 0 0 1 1 %d\n", 
                 username, size, id);
              break;	 
      case 4: printf("UUU %s "SI64" 0 0 0 0 1 %d\n", 
                 username, size, id);
              break;
   }
   if (!fgets(str, sizeof(str), stdin)) strlcpy(str, "-pipe error", sizeof(str));
   if (*str == '+') return 0;
   else return -1;
}


/* nuke all files in a directory and all subdirs below 
   path = relative to basedir, nukelog = pointer to open log file
   file + totalsize = output pointers
 */
int nuke_files_r(char* path, float factor, FILE *nukelog, int section_id, 
                 int *files, gint64 *totalsize)
{ 
   struct dirent* d;
   DIR*           dir;
   char*	  username;
   char*          groupname;
   gint32          gid = 0;
   gint32          uid = 0;
   gint32          perm;    
   struct stat    st;
   char		  str[256];
   char           fullpath[1024];
   int            times;
   int            err = 0;
   int            hidden;

   dir = opendir(path+1);
   if (!dir) return 0;
   while ((d = readdir(dir)) != NULL) { 
      g_snprintf(fullpath, sizeof(fullpath), "%s/%s", path, d->d_name);
      if ((d->d_name[0] != '.') && !stat(fullpath+1, &st) && 
         strcmp(d->d_name, DFL_NUKELOG)) {
         if (S_ISDIR(st.st_mode)) {       	
	    err = nuke_files_r(fullpath, factor, nukelog, section_id, files, 
	       totalsize);
         } 
         else if (st.st_size > 0) { 
	    if ((permission_ok(path, PE_CHDIR) != -1) && 
	       (read_file_permission(path, d->d_name, &uid, &gid, &perm) != -1)) 
	    {
	       username  = get_username(uid);
               groupname = get_groupname(gid);
               (*totalsize) += st.st_size;
 	             
	       g_snprintf(str, sizeof(str), " !C|!H %-10.10s %7ld KB   %-46.46s !C|!0", 
		  username, (long) (st.st_size / 1024), d->d_name);
	       send_message(211, 0, str, NULL);
	       fprintf(nukelog, "%-10.10s %-10.10s %9.0f /%s\n", username,
		  groupname, (float) st.st_size, fullpath);
               hidden = directory_flag_set_recursive(path, DP_HIDDEN);
               log_action(hidden, LG_MISC, "NUKE","%s (%f * %lu) %s",
                  d->d_name, factor, st.st_size, username);                            

	       /* remove credits and update nukestats */
	       printf("UN %s "SI64" 1 %f %d\n", username, (gint64) st.st_size,
	          factor, section_id);
               if (!fgets(str, sizeof(str), stdin)) strlcpy(str, "-pipe error", sizeof(str));
               if (*str == '+') err = 0;
               else err = 1;

	       /* change the stats according to the file age            *
		* the times variable is needed to decided which of the  *
		* stats (day/week/month/year/total) to change           */
               times = get_time(st.st_mtime);
               if (factor > 0) update_stats(username, section_id, st.st_size, times);
	       else update_stats(username, section_id, st.st_size * -1, times);
            } 
	    else {
	       err = 1;
            }
	 } 
      }             
   }
   closedir(dir);
   return err;
}  


/* nuke a single file */
int nuke_file(char* relpath, float factor, char* username, gint64 filesize,
              int times)
{
   char	   str[256]; 
   int     section_id;
   int     hidden;
 
   /* get section id */
   printf("SG_ID %s\n", relpath);
   if (!fgets(str, sizeof(str), stdin)) {
      send_message(550, 0, "!Bpipe error!0", NULL);
      return -1;
   }
   else if (*str == '-') {
      send_message(550, 0, str + 1, NULL);
      return -1;
   }
   else if (!sscanf(str + 1, "%d", &section_id)) {
      send_message(550, 0, "!BUnable to get section id!0", NULL);
      return -1;
   }

   print_nuke_head();
   g_snprintf(str, sizeof(str), " !C|!H %-10.10s %7"I64" KB   %-46.46s !C|!0", 
      username, (gint64) filesize / 1024, relpath);
   send_message(211, 0, str, NULL);
   print_nuke_foot(filesize, factor);
   hidden = directory_flag_set_recursive(relpath, DP_HIDDEN);
   log_action(hidden, LG_MISC, "NUKE","%s (%f * %lu) %s", relpath,
      factor, filesize, username);

   printf("UN %s "SI64" 1 %f %d\n", username, (gint64) filesize, 
      factor, section_id);
   if (!fgets(str, sizeof(str), stdin)) {
      send_message(200, 0, "!Bpipe error!0", NULL);
   }
   else if(*str == '-') {
      send_message(550, 0, "!BError updating user %s!0", username);      
   }
   update_stats(username, section_id, (factor > 0) ? (gint64) filesize : 
      (gint64) filesize * -1, times);

   printf("UN %s 0 0 %f %d\n", username, factor, section_id);
   if(!fgets(str, sizeof(str), stdin)) {
      send_message(200, 0, "!Bpipe error!0", NULL);
   }
   else if(*str == '-') {
      send_message(550, 0, str+1, NULL);
      return -1;
   }
   
   /* We remove when nuking a single file */
   if (remove(relpath+1) == -1) {
      send_message(550, 0, "!BUnable to delete file %s!0", relpath);
      return -1;
   }
   return 0;
}


int nuke_directory(char *fullpath, char* relpath, float factor, char *reason, 
                   char *username, char *groupname)
{
   char           str[256];
   char           str2[256];
   struct tm      *tm;
   time_t         t;
   FILE           *nukelog;
   char           timestr[24];
   char           filename[256];
   gint64          totalsize = 0;
   int            files = 0;
   int            section_id;
   char           *dir, *name;
   gint64          emptysize; /* size to nuke for a empty dir */
   gint32          del_empty_nuke = 0; /* bool value */
   char           nukeprefix[32];

   time(&t); 
   tm = localtime(&t); 
   strftime(timestr, 23, "%Y-%m-%d %H:%M:%S", tm);

   /* ---- get empty nuke size to penalice ---------------------- */
   printf("SGV_DIR empty_nuke %s\n", relpath);
   if (!fgets(str, sizeof(str), stdin)) return -1;
   if (*str != '+' || !sscanf(str+1, SI64, &emptysize)) {
      emptysize = DFL_EMPTYSIZE;
   }
   emptysize *= 0x100000;

   /* ---- get section id   ------------------------------------- */
   printf("SG_ID %s\n", relpath);
   if (!fgets(str, sizeof(str), stdin)) strlcpy(str, "-pipe error", sizeof(str));
   if (*str != '+' || 1 != sscanf(str+1, "%d", &section_id)) {
      section_id = 0;
   }

   /* ---- get del empty nuked dirs flag ------------------------ */
   printf("SGV_DIR del_empty_nuke %s\n", relpath);
   if (!fgets(str, sizeof(str), stdin)) strlcpy(str, "-pipe error", sizeof(str));
   if (*str != '+' || !sscanf(str + 1, SI32, &del_empty_nuke)) {
      del_empty_nuke = 1;
   }

   /* ---- get nuke prefix -------------------------------------- */
   printf("SGV_DIR nukeprefix %s\n", relpath);
   if (!fgets(str, sizeof(str), stdin)) strlcpy(str, "-pipe error", sizeof(str));
   if (*str != '+' || !sscanf(str+1, "%s", nukeprefix)) {
      strlcpy(nukeprefix, DFL_NUKEPREFIX, sizeof(nukeprefix));
   }

   g_snprintf(filename, sizeof(filename), "%s/%s", fullpath, DFL_NUKELOG);
   nukelog = fopen(filename, "wt");
   if (!nukelog) return -1;
   
   fprintf (nukelog, "nuked by %s/%s at %s factor: %.0f reason: %s\n\n",
      usr.name, get_groupname(usr.gid), timestr, factor, reason);

   strlcpy(str, factor > 0 ? "Nuking !H%s!0 ..." : "Unnuking !H%s!0 ...", sizeof(str));
   send_message(211, 0, str, relpath);

   print_nuke_head();
   /* get total size of all files by calling recursive function */
   nuke_files_r(relpath, factor, nukelog, section_id, &files, &totalsize);
   print_nuke_foot(totalsize, factor);

   /* ---- empty directory (nukee = directory owner) ------------ */
   if (totalsize == 0) { 
      printf("UN %s " SI64 " 0 %f %d\n", username, emptysize, factor, section_id);
      if (!fgets(str, sizeof(str), stdin)) strlcpy(str, "-pipe error", sizeof(str));
      if (*str != '+') return -1;
       
      g_snprintf(str, sizeof(str), "Empty directory found. !H" SI64 " MB!0 credits !H%s!0.",
         emptysize / 0x100000, (factor > 0) ? "removed" : "re-added");
      send_message(211, 0, str, NULL);

      fprintf(nukelog, "%-10.10s %-10.10s "SI64" -empty-\n", username,
		  groupname, (gint64) emptysize);

      /* script does deleting now 
	 if (factor > 0) {
            if (del_empty_nuke) {
               strncpy(str, fullpath, sizeof(str) - 1);
               del_files_r(str); 
               rmdir(fullpath);
               strncpy(str, "Directory was deleted successfully.", sizeof(str) - 1);
            }
	 }
	 else strncpy(str, "Empty directory not deleted in this section.", sizeof(str) - 1);
	 send_message(211, 0, str, NULL);
      */
   }
   fclose(nukelog);

   /* ---- rename the directory --------------------------------- */
   strlcpy(str2, fullpath, sizeof(str2));
   split_dir(str2, &dir, &name, 0); /* destuctive */

   if (factor < 0) {
      /* rename "_NUKED_Release" to "Release" (unnuke) */
      g_snprintf(str, sizeof(str), "%s/%s", dir, name + strlen(nukeprefix));           
   }
   else {
      /* rename "Release" to "_NUKED_Release" (nuke) */
      g_snprintf(str, sizeof(str), "%s/%s%s", dir, nukeprefix, name);
   }

   if (rename(fullpath, str) == -1) {
      send_message(211, 0, "!BUnable to rename directory!0", NULL);
      return -1;
   }
   
   /* ---- add 1 to the times nuked counter --------------------- */
   printf("UN %s 0 0 %f %d\n", username, factor, section_id);
   if (!fgets(str, sizeof(str), stdin)) strlcpy(str, "-pipe error", sizeof(str));
   if (*str != '+') return -1;

   /* ---- call the nuke / unnuke script ------------------------ */
   g_snprintf(str2, sizeof(str2), "\"%s\" \"%s\" %d", fullpath, nukeprefix, 
      totalsize == 0 ? del_empty_nuke : 0);
   if(factor > 0) send_message(211, 0, nuke_msg.msg, str2);
   else send_message(211, 0, unnuke_msg.msg, str2);

   return 0;
}


int cmd_site_nuke(char** p, int n)
{ 
   char		  str[256]; 
   char           basedir[256];      /* path to site dir                 */
   float          factor = 1.0;
   struct stat    st;
   char           relpath[512];      /* dir to nuke relative to site dir */ 
   char           fullpath[1024];    /* full path to the nuke directory  */
   int            err = 0;
   gint32          gid = -1;
   gint32          uid = -1; 
   gint32          perm = 0; 
   gint32          dlc = 0;    
   char           *dir,  *name; 
   char           *username;         /* owner of the file or directory   */  
   char           *groupname;        /* group of the owner               */

   if (!nuker(usr.name) && !superuser()) {
      return send_reply(550, "!BYou don't have nuke permission.!0");
   }

   if (n != 3) {
      send_message(501, 0,"!BWrong number of arguments!0", NULL);
      return cat_help("nuke", 501); 
   }   

   /* create absolute and relative path */ 
   getcwd(basedir, sizeof(basedir));
   add_dir(vfs_getcwd (), p[0], relpath, sizeof(relpath));
   g_snprintf(fullpath, sizeof(fullpath), "%s%s", basedir, relpath);

   if (stat(fullpath, &st) == -1) {
      return send_reply(550, "!BUnable to open directory or file!0");
   }
   if (!sscanf(p[1], "%f", &factor)) {
      send_message(501, 0, "!BInvalid nuke factor!0", NULL);
      return cat_help("nuke", 501); 
   }
   if (factor == 0) {
      return send_reply(501, 
         "!BFactor 0 is useless. Use SITE DEL to remove without nuking.!0");
   }   
   if (factor > 20) {
      return send_reply(501, 
         "!BFactor is too high, probably you made a mistake.!0");
   }

   strlcpy(str, relpath, sizeof(str));
   split_dir(str, &dir, &name, 0);   /* destructive function! */
   if (permission_ok(vfs_getcwd (), PE_CHDIR) == -1) {
     return send_message(550, 1, "!BUnable to access directory %s!0", vfs_getcwd() );
   }

   /* nuking a directory */
   if (S_ISDIR(st.st_mode)) {           
      if (read_permission_file(relpath, &uid, &gid, &perm, &dlc, NULL, 0) == -1) {
	 return send_message(550, 1, "!BUnable to read permissions for %s!0", name);
      }
      username = get_username(uid);
      groupname = get_groupname(gid);

      if(strncasecmp(kickonnuke,"ALL",3)==0)
      {
        errlog("Kicking ALL users from nuked directory");
      }
      else if(strncasecmp(kickonnuke,"TRANSFERS",8)==0)
      {
        errlog("Kicking all users transferring files in nuked directory");
      }

      if (nuke_directory(fullpath, relpath, factor, p[2], username, 
         groupname) == -1) err = 1;
   }
   /* nuking single file */
   else {
      if (read_file_permission(dir, name, &uid, &gid, &perm) == -1) {
	 return send_message(550, 1, "!BUnable to read permissions for %s!0", name);
      }
      username = get_username(uid);
      groupname = get_groupname(gid);

      if (nuke_file(relpath, factor, username, st.st_size, 
         get_time(st.st_mtime)) == -1) err = 1;
   }

   if (err) { 
      return send_message(211, 1, "!BThere were errors.!0", NULL);
   }
   else {
      return send_message(211, 1, factor > 0 ? 
         "!HNuke successful.!0" : "!HUnnuke successful.!0", NULL);   
   }
}	


int cmd_site_unnuke(char** p, int n)
{ 
   char		 str[256];
   char*         p2[3];         /* new parameter vector */
   struct stat   st;
   FILE          *nukelog;
   char          relpath[512];      /* dir to nuke relative to site dir */ 
   char          fullpath[1024];    /* full path to the nuke directory */
   float         f;
   char          factor[10]; 

   if(!nuker(usr.name) && !superuser()) {
      return send_reply(550, "!BYNo permission!0");
   }
   if (n != 2) {
     send_message(501,0,"!BWrong number of arguments!0",NULL);
     return cat_help("unnuke", 501); 
   }

   /* create absolute and relative path */ 
   getcwd(str, sizeof(str));
   add_dir(vfs_getcwd (), p[0], relpath, sizeof(relpath));
   g_snprintf(fullpath, sizeof(fullpath), "%s%s/%s", str, relpath, DFL_NUKELOG);

   if (stat(fullpath, &st) == -1) {
      return send_reply(550, "!BUnable to open directory or file!0");
   }

   nukelog = fopen(fullpath, "rt");       
   if (nukelog) {
      fgets(str, sizeof(str), nukelog);
      sscanf(strstr(str, "factor"), "factor: %f", &f);
      fclose(nukelog);
   }
   else return send_reply(550, "!BUnable to open nukelog!0");

   g_snprintf(factor, sizeof(factor), "%.1f", f * -1);
   
   p2[0] = p[0];
   p2[1] = g_malloc(sizeof(factor));
   strcpy(p2[1], factor);
   p2[2] = p[1];
   return cmd_site_nuke(p2, 3);
}



/* SITE GIVE command (transfer credits to another user) */
int cmd_site_give(char** p, int n)
{ 
   char str[128];
   gint64 credits;

   if (!sitegive) 
      return send_reply(550, "!BCommand disabled!0");
   if (n != 2) return cat_help("give",501); 
   if (!sscanf(*(p+1),SI64,&credits)) return cat_help("give",501);
   if (get_uid(p[0]) == -1) return send_reply(550, "!BUnknown user!0");

   if(superuser()) {
       printf("UCR %s %"I64"\n" ,p[0], credits * -1);
       if (!fgets(str,sizeof(str),stdin)) strlcpy(str, "-!Bpipe error!0", sizeof(str));
       if (*str == '+') {
          g_snprintf(str, sizeof(str), "Gave %"I64" MB credits to %s", credits, p[0]);
          return send_reply(200, str);
       }   
       else return send_reply(550,str+1);	  	  
   }
   else {
      if(strcmp(usr.name, p[0]) == 0) return send_reply(550, "!BDamn cheater, you can't steal credits!!!0");
      if (credits < 0) return send_reply(550, "!BDamn cheater, you can't steal credits!!!0");
      if(usr.credit >= (credits*1048576)) {           
       	  printf("UCR %s %"I64"\n" ,p[0], credits * -1);
          if (!fgets(str,sizeof(str),stdin))
             strlcpy(str, "-!Bpipe error!0", sizeof(str));
          if (*str == '+') {
             send_message(200, 0, str+1, NULL); 
          } else return send_reply(550, "!BUnable to transfer credits!0");

          printf("UCR %s %"I64"\n" ,usr.name, credits);         
          if (!fgets(str,sizeof(str),stdin))
             strlcpy(str, "-pipe error", sizeof(str));
          if (*str == '+') {
             g_snprintf(str, sizeof(str), "Gave %"I64" MB credits to %s", credits, p[0]);
             send_reply(200, str); 
          } else return send_reply(550, "!BUnable to transfer credits!0");
      }
      else return send_reply(550, "!BYou don't have enough credits!0");
   }
   return 0;
}


/* SITE TAKE command (remove credits from an user) */
int cmd_site_take(char** p, int n)
{ 
   char    str[128];
   gint64   credits = 0;

   if(!superuser()) return send_reply(550, "!BYou are not allowed to use this command.!0");
   if (n != 2) return cat_help("take",501);
   if (!sscanf(*(p+1),"%"I64"",&credits)) cat_help("take",501);
   if (get_uid(p[0]) == -1) return send_reply(550, "!BUnknown user!0");

   if(!superuser()) return send_reply(550, "!BYou are not allowed to use this command.!0");
   printf("UCR %s %"I64"\n" ,p[0], credits);
   if (!fgets(str,sizeof(str),stdin)) strlcpy(str, "-pipe error", sizeof(str));
   if (*str == '+') {
      g_snprintf(str, sizeof(str), "Took !H%"I64" MB!0 credits from !H%s!0", credits, p[0]);
      return send_reply(200, str);
   }   
   else return send_reply(550,str+1);
}  


/* EOF */
