/*
    $Id: checksum.c,v 1.6 2001/02/17 13:54:25 hvr Exp $

    Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <commondef.h>

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <glib.h>

#include "crc32.h"

#define BUFSIZE (32*1024)

static guint32
crc32_calc_file (const gchar fname[])
{
  FILE *in;
  gchar *buf;
  guint32 crc = 0;
   
   if((in = fopen(fname, "rb")) == NULL)
     return 0;

   buf = g_malloc(BUFSIZE);

   while (!feof (in))
     {
       gint bytes = fread (buf, 1, BUFSIZE, in);
       
       crc = crc32_calc (crc, buf, bytes);
     }
   
   g_free (buf);

   fclose(in);
   
   return crc;
}

int main(int argc, char *argv[]) 
{
  if (argc == 2)
    {
      guint32 crc = crc32_calc_file (argv[1]);
      
      fprintf (stdout, "%08X\n", crc);   
      
      exit (EXIT_SUCCESS);
    }
      
  fprintf (stderr, "%s: wrong number of arguments\n", argv[0]);
  exit (EXIT_FAILURE);
}
