/* getpw 0.1 by soulshock
 * utility to fetch passwords from openftpd userdatabase
 * Syntax: getpw <path_to_userfile> <username> [password]
 * If password is supplied, returns 0 if match and 1 if not
 * If no password is supplied, prints password and returns 0
 */
#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <time.h>
#include <string.h>
#include <unistd.h>
#ifdef CRYPT_CRYPT_H
#include <crypt.h>
#else
#ifdef CRYPT_UNISTD_H
#include <unistd.h>
#endif /* #ifdef CRYPT_UNISTD_H */
#endif /* #ifdef CRYPT_CRYPT_H */

typedef signed long int32;
typedef signed long long int64;

struct user_s
{ 
   int32     uid;            /* user ID                              */
   int32     gid;            /* group ID                             */
   int32     gids[8];        /* additional groups                    */
   int32     class;          /* user class                           */
   int64     rights;         /* user rights                          */
   int32     limit;          /* limit (ratio)                        */
   int64     credit;         /* credit in bytes                      */
   int32     dl_speed;       /* min and max download speed in kb/s   */
   int32     ul_speed;       /* min and max upload speed   in kb/s   */
   int64     dl_week;        /* download current week                */
   int64     ul_week;        /* upload current week                  */
   int64     dl_month;       /* download current month               */
   int64     ul_month;       /* upload current month                 */
   int64     dl_year;        /* download current year                */
   int64     ul_year;        /* upload current year                  */
   int64     dl_total;       /* download total                       */
   int64     ul_total;       /* upload total                         */
   int64     dl_tm_week;     /* download time (s) current week       */
   int64     ul_tm_week;     /* upload time (s) current week         */
   int64     dl_tm_month;    /* download time (s) current month      */
   int64     ul_tm_month;    /* upload time (s) current month        */  
   int64     dl_tm_year;     /* download time (s) current year       */
   int64     ul_tm_year;     /* upload time (s) current year         */
   int64     dl_tm_total;    /* download time (s) total              */
   int64     ul_tm_total;    /* upload time (s) total                */
   time_t    last_login;     /* time of last login                   */
   char      name[10];       /* username                             */
   char      password[40];   /* encrypted password                   */
   char      info[32];       /* info                                 */
   char      idents[10][20]; /* 12bytes for ident; 8byte for iprange */
   int64     dl_day;         /* bytes downloaded today               */
   int64     ul_day;         /* bytes uploaded today                 */
   int64     dl_tm_day;      /* download time (s) today              */
   int64     ul_tm_day;      /* upload time (s) today                */
   int32     ul_f_day;       /* files uploaded today                 */
   int32     dl_f_day;       /* files downloaded today               */ 
   int32     ul_f_total;     /* files uploaded total                 */
   int32     dl_f_total;     /* files downloaded total               */
   int32     ul_f_week;      /* files uploaded current week          */
   int32     dl_f_week;      /* files downloaded current week        */
   int32     ul_f_month;     /* files uploaded current month         */
   int32     dl_f_month;     /* files downloaded current month       */
   int32     ul_f_year;      /* files uploaded current year          */
   int32     dl_f_year;      /* files downloaded current year        */
   int32     f_nuked_total;  /* files nuked total                    */
   int64     b_nuked_total;  /* bytes nuked total                    */
   int64     c_nuked_total;  /* credits nuked total                  */
   int32     t_nuked_total;  /* times nuked total                    */
   int32     f_nuked_week;   /* files nuked current week             */
   int64     b_nuked_week;   /* bytes nuked current week             */
   int64     c_nuked_week;   /* credits nuked current week           */
   int32     t_nuked_week;   /* times nuked current week             */
   int32     f_nuked_month;  /* files nuked current month            */
   int64     b_nuked_month;  /* bytes nuked current month            */
   int64     c_nuked_month;  /* credits nuked current month          */
   int32     t_nuked_month;  /* times nuked current month            */ 
   int32     f_nuked_year;   /* files nuked current year             */
   int64     b_nuked_year;   /* bytes nuked current year             */
   int64     c_nuked_year;   /* credits nuked current year           */
   int32     t_nuked_year;   /* times nuked current year             */
   int32     f_nuked_day;    /* files nuked today                    */
   int64     b_nuked_day;    /* bytes nuked today                    */
   int64     c_nuked_day;    /* credits nuked today                  */
   int32     t_nuked_day;    /* times nuked today                    */
   time_t    creation_time;  /* creattion time of the account        */
   char      creator[10];    /* the creator of the account           */
   int64     wkly_alot;      /* weekly alottment to add credits      */
   /*int64     temp[32];        temp buffer for future uses          */
   char      home[256];      /* this should work unless size(char) != 1 byte */ 
};

struct uf_ident_s
{ char id[32];
  int  version;
  int  count;
  int  encrypted;
  int  week;
  int  month;
  int  year;
  int  day; /* This was temp1 changed now to day */
  int  temp2;
} u_id = { "OpenFTPD user database", 101, 0, 0, 0, 0, 0, 0, 0 }; 

int main (int argc, char *argv[]) {
  char *userfile;
  int ret;
  FILE *UFILE;
  struct user_s user;
  struct uf_ident_s header;

  if(argc < 3) {
    printf("Usage: %s <path_to_userfile> <username> [password]\n", *argv);
    printf("If password is supplied, program will return 0 if it matches and 1 if not\n");
    printf("If no password is supplied, program will print users password\n");
    exit(0);
  }
  userfile = argv[1];

  /* Skip the header, since we dont care about this */
  /* We'll just assume latest openftpd (0.30+) */
  if((UFILE = fopen(userfile, "r")) == NULL) {
    perror("error opening userfile");
    exit(2);
  }
  ret = fread(&header, sizeof(header), 1, UFILE);
  if(ret < 1) {
    perror("error reading header");
    fclose(UFILE);
    exit(2);
  }

  /* Start parsing file */
  while(!feof(UFILE)) {
    ret = fread(&user, sizeof(user), 1, UFILE);
    if(ret < 1) {
      perror("error reading file");
      fclose(UFILE);
      exit(2);
    }
    /* User found. */
    if(strncmp(user.name, argv[2], strlen(user.name)) == 0) {
      if(argc == 4){
	/* Encrypt password and compare it to the one, from the userfile */
	if(strcmp((char *)crypt(argv[3], user.password), user.password)) {
	  /* Naw didnt fit */
	  fclose(UFILE);
	  printf("NO MATCH!\n");
	  exit(1);
	} else {
	  /* We have a winner! */
	  fclose(UFILE);
	  printf("MATCH!\n");
	  exit(0);
	}
      }
      /* No password was supplied, so we'll just output password from userfile and quit */
      printf("%s\n", user.password);
      break;
    }
  }

  fclose(UFILE);
  return 0;
}
