#!/usr/bin/perl -w

##############################################################################
#                                                                            #
#  primeCHECK v2.3 - UNDUPE (c) 2000 primemover                              #
#                                                                            #
#  undupe for directory dupelist                                             #
# ---------------------------------------------------------------------------#
#                                                                            #
#  This program is free software; you can redistribute it and/or modify      #
#  it under the terms of the GNU General Public License as published by      #
#  the Free Software Foundation; either version 2 of the License, or         #
#  (at your option) any later version.                                       #
#                                                                            #
##############################################################################


# ----------------------------------------------------------------------------
#  number of days to keep the dupelog
# ----------------------------------------------------------------------------
my $maximumdays = 30;


# ----------------------------------------------------------------------------
#  logfiles
# ----------------------------------------------------------------------------
my $directorylog = $standarddir."var/log/directory.log";



##############################################################################
# end of configuration (don't edit below unless you know what you are doing  #
##############################################################################

use strict;
use Cwd;
use Time::Local;
use File::Basename;

use FindBin;
use lib "$FindBin::Bin/";
use primetools;


##############################################################################
# main program                                                               #
##############################################################################

my $syntax = "DIRUNDUPE: remove directories from the dupelist\n" .
          "Syntax: undupe check|nocheck <string>\n" .
	  "Example: undupe nocheck dir xyz\n";

if ($#ARGV < 1 or $ARGV[0] !~ /check|nocheck/i) { 
   print $syntax; 
   exit; 
}

# use a 1st parameter like "nocheck" to disable the permission check
if ($ARGV[0] eq "check") {
   if ((exists $ENV{F4A_R_UNDUPE} and $ENV{F4A_R_UNDUPE} ne "yes") and 
       (exists $ENV{F4A_CLASS} and $ENV{F4A_CLASS} !~ /^(s|o)$/)
      ) {
      print "You are not allowed to use the SITE UNDUPE command. Ask your admin :)\n";
      exit 0;
   }
}

open(DIRLOG, "< $directorylog") or die "ERROR: unable to open $directorylog\n";
open(NEWDIRLOG, "> $directorylog.tmp") or 
die "ERROR: unable to open $directorylog.tmp\n";
while (<DIRLOG>) {
   my $dir = ((split(/\s+/, $_))[5]);
   if (defined $dir) {
      $dir = basename($dir);
   }
   my $tm  = stringtotime($_);
   my $dir2 = basename($ARGV[1]);
   if ($dir !~ /\Q$dir2\E/i and (time - $tm < $maximumdays * 86400)) { 
      print NEWDIRLOG $_; 
   }
}
close(DIRLOG);
close(NEWDIRLOG);
rename "$directorylog.tmp", $directorylog;
print "All matching entries were removed from dupe log.\n";

# --- end of file ---
