#!/usr/bin/perl -w

##############################################################################
#                                                                            #
#  OpenFTPD Fill-Request Script v1.0 - (c) 2000 topace & primemover          #
#                                                                            #
# ---------------------------------------------------------------------------#
#                                                                            #
#  This program is free software; you can redistribute it and/or modify      #
#  it under the terms of the GNU General Public License as published by      #
#  the Free Software Foundation; either version 2 of the License, or         #
#  (at your option) any later version.                                       #
#                                                                            #
##############################################################################

use strict; #always

our $standarddir   = $ENV{'F4A_STD'};
our $sitename      = "XYZ";
our $botpidfile    = $standarddir."/../sitebot/pid.Jackie_B";
our $announcefile  = $standarddir."/../sitebot/announce.txt";
our $requestlog    = $standarddir."/var/log/request.log";

use FindBin;
use lib "$FindBin::Bin/";
use primetools;

#Global variables - hmm
our ($requestdir,$username,$groupname,$forusername,$forgroupname);

# ----------------------------------------------------------------------------
#   announce_newdir() - what do you think is this doing ? 
# ----------------------------------------------------------------------------
sub announce_fillreq {
   my $formatfile = $standarddir."msg/irc/fillrequest";
   my $formatstring = "";

   open(FORMATFILE, "< $formatfile") or return 0;
   $formatstring = <FORMATFILE>;
   close(FORMATFILE);
   chomp($formatstring);

   (my $minutes, my $hours) = (localtime)[1..2];
   my $time = sprintf("%02d:%02d", $hours, $minutes);

   for ($formatstring) {
      s/<b>|<\/b>/$b/g;
      s/<u>|<\/u>/$u/g;
      s/\$sitename/$sitename/g;
      s/\$directory/$requestdir/g;
      s/\$username/$username/g;
      s/\$groupname/$groupname/g;
      s/\$forusername/$forusername/g;
      s/\$forgroupname/$forgroupname/g;
      s/\$time/$time/g;
   }
   announce_line($announcefile, $botpidfile, $formatstring);
}


###############################################################################
# main program                                                                #
###############################################################################

my ($void, $basedir, $permfile,$requestbasedir,$requestnum,$ok,$rnum,$perm);
if ($#ARGV < 0 ) {
  print " !HUsage: !Csite fillrequest <RequestNumber>!0\n",
        " !HDo !Csite request !Hto get a list of requests!0";
  exit 0;
}

# read the ftp4all username and groupname from environment variables
if (exists $ENV{'F4A_USER'}) { $username = $ENV{'F4A_USER'} }
else { $username = "unknown" };
if (exists $ENV{'F4A_GROUP'}) { $groupname = $ENV{'F4A_GROUP'} }
else { $groupname = "unknown" };

#read registry and dircfg to global hashes
my %registry = read_registry(); 

if (exists $registry{botpidfile}) { 
  $botpidfile = $registry{botpidfile}; 
}
if (exists $registry{announcefile}) { 
  $announcefile = $registry{announcefile}; 
}
if (exists $registry{basedir}) { 
  $basedir= $registry{basedir}; 
}
if (exists $registry{shortsitename}) {
  $sitename = $registry{shortsitename};
}
$permfile = ".permissions";
if (exists $registry{permissionfile}) {
  $permfile = $registry{permissionfile};
}

# FIRST, lets make sure there is a request directory defined, 
# if not, cant fill requests!
open(SECTIONSCFG,"< ${standarddir}/etc/sections.cfg");
while (<SECTIONSCFG>) {
   if ($_ =~ /REQUEST/i) {
      ($void, $void, $void, $requestbasedir) = split(/\s+/,$_,4); 
      chomp($requestbasedir);
   }
}
close(SECTIONSCFG);
if (!$requestbasedir) {
   print " There is NO REQUEST directory defined in sections.cfg, ",
      "please contact siteop!";
   exit 0;
} 

$requestnum = $ARGV[0];
printf(" Filling request number %d\n", $requestnum);
$ok = 0;
if (-s $requestlog) {
   if (open (REQFILE,"< $requestlog")) {
      $rnum=0;
      while (<REQFILE>) {
         chomp($_);
         if ($_ and $rnum == $requestnum) {
            ($void, $forusername, $forgroupname, $requestdir) = 
	       split(/\s+/, $_, 4);
            $ok = 1; 
         }
         $rnum++;
      }
      close(REQFILE);
   }
}

if ($forusername eq $username) {
   print "!B You are not allowed to fill your own requests!!!0\n";
   exit 0;
}

if (not -d "$basedir/$requestbasedir") {
   print " Request directory was not found. Please contact siteop!\n";
   exit 0;
}

if ($ok) {
   open(FILE, "< $requestlog");
   open(OUTFILE, "> ${requestlog}.tmp");
   while (<FILE>) {
      if ($_ !~ /$requestdir/) {
         print OUTFILE $_;
      }
   }
   close(FILE);
   close(OUTFILE);
   rename ("${requestlog}.tmp", "$requestlog");

   $perm = pack "i2H8i9", 0, 0, "37377F68", 0, 0, 0, 0, 0, 0, 0, 0, 0;
   mkdir("$basedir/$requestbasedir/$requestdir", 0700);
   open(PERMFILE,"> $basedir/$requestbasedir/$requestdir/$permfile");
   print PERMFILE $perm;
   close(PERMFILE);

   announce_fillreq();
   print " !hFillrequest ok, upload into !H$requestbasedir/$requestdir!0\n";
}
else {
   print " !BAn ERROR occured, please contact siteop!0\n";
}
exit 0;

# --- end of file ---
