#!/usr/bin/perl -w

##############################################################################
#                                                                            #
#  primeLAST v2.1.1 (c) 2000 primemover                                      #
#                                                                            #
#  show the latest directories on your OpenFTPD site                         #
# ---------------------------------------------------------------------------#
#                                                                            #
#  This program is free software; you can redistribute it and/or modify      #
#  it under the terms of the GNU General Public License as published by      #
#  the Free Software Foundation; either version 2 of the License, or         #
#  (at your option) any later version.                                       #
#                                                                            #
##############################################################################

 
my $standarddir   = $ENV{'F4A_STD'};

# ----------------------------------------------------------------------------
#  the directory var/log/dupe file
# ----------------------------------------------------------------------------
my $directorylog = $standarddir."var/log/directory.log";


# ----------------------------------------------------------------------------
#  show uploads that were already deleted ?
# ----------------------------------------------------------------------------
my $showdeleted   = 0;


# ----------------------------------------------------------------------------
#  ignore all directories matching this regexpr
# ----------------------------------------------------------------------------
my $ignore = 'cd[0-9]|sample|cover.*|-\[.+|\[.+';


# ----------------------------------------------------------------------------
#  default number of lines
# ----------------------------------------------------------------------------
my $lines = 10;


my $statusfile  = ".status";

my $body =
" !C| !c[!C!H%-48.48s !c]-[!CSECT: !H%-11.11s!c]!C|!0\n" . 
" !C| !c[!CAGE: !H%7.7s!c]-[!COWNER: !H%-10.10s!c]-[!CSTATUS: !H%2.2s" .
"!c/!H%2.2sF!c]-[!CSIZE: !H%8d MB!c]!C|!0\n" .
" !C|!c" . " " x 72 . "!C|!0\n";


use strict; # always
use Cwd;
use Time::Local;
use File::Basename;
use File::Find;
use Getopt::Std;

use FindBin;
use lib "$FindBin::Bin/";
use primetools;

# ----------------------------------------------------------------------------
#  strdifftime($earlier, $later) - calc the diff between two times as string
# ----------------------------------------------------------------------------
sub strdifftime {
   my $time1   = $_[0]; my $time2 = $_[1];
   my $diffstr = "";  my $diff = $time2 - $time1;
   my $seconds = $diff % 60; $diff = ($diff - $seconds) / 60;
   my $minutes = $diff % 60; $diff = ($diff - $minutes) / 60;
   my $hours   = $diff % 24; $diff = ($diff - $hours) / 24;
   my $days    = $diff % 7; 
   my $weeks   = ($diff - $days) / 7;

   if ($weeks != 0) { 
      $diffstr .= sprintf("%02dw", $weeks); 
      if ($days or $hours or $minutes or $seconds) {
         $diffstr .= " ";
      }
   }
   if ($days  != 0) { 
      $diffstr .= sprintf("%02dd", $days);  
      if ($hours or $minutes or $seconds) {
         $diffstr .= " ";
      }
   }
   if ($hours != 0 and $weeks == 0) { 
      $diffstr .= sprintf("%02dh", $hours); 
      if ($minutes or $seconds) {
         $diffstr .= " ";
      }	 
   }
   if ($minutes != 0 and $days == 0 and $weeks == 0) { 
      $diffstr .= sprintf("%02dm", $minutes); 
      if ($seconds) {
         $diffstr .= " ";
      }	      
   }
   return $diffstr;
}


###############################################################################
# main program                                                                #
###############################################################################

my %args;
my %registry = read_registry();
my @lastdirs = ();

getopt("s:l:", \%args);

my $filter = ".+";
if (exists $args{s}) {
   $filter = $args{s}; 
}

if (exists $args{l}) {
   $lines = $args{l}; 
}


my $footer = 
' !C|!c------------------------------------------------------------------------!C|!0 ' . "\n" .
' !C| !hUse SITE NEW [-l <lines>] [-s <section>] for a filtered view.          !C|!0 ' . "\n" .
' !C`------------------------------------------------------------------------+ !0' . "\n";


my $header = 
' !A     ____        ______     ______     ____                               !0 ' . "\n" .
' !C.-!A _/   |_____ _/  _   \_ _/  ___/__ _/   |_ ____ !C-----------------------.!0 ' . "\n" .
' !C|!h _\    |    /_\   _    /_\____    /_\    _/    /_                       !C|!0 ' . "\n" .
' !C|!h |     |     |    |     |    |     |     |      |  !HLAST UPLOADED        !C|!0 ' . "\n" .
' !C|!H |______     |____|     |____      |______   !Apm !H|  !HDIRECTORIES          !C|!0 ' . "\n" .
' !C|!H       \_____|    \_____|   \______|     \______|                       !C|!0 ' . "\n" .
' !C|!c------------------------------------------------------------------------!C|!0 ' . "\n";

printcolor($header); 

open(DIRLOG, "< $directorylog") or die "ERROR unable to open directory log\n";
while (<DIRLOG>) {
   chomp($_);
   if($_ and $_ !~ /$ignore/i) { 
      my $section = (split(/\s+/, $_, 6))[4];
      if ($section =~ /$filter/i) {
	 push(@lastdirs, $_); 
      }
      if ($#lastdirs >= $lines) { 
	 shift(@lastdirs); 
      }   
   }
}
close(DIRLOG);

my $number = 0;

foreach my $lastdir (reverse @lastdirs) {
   my $totalsize  = 0;
   my $filecount  = 0;
   my $totalfiles = 0;
   my $goodfiles  = 0;
   
   my ($username, $section, $directory) = (split(/\s+/, $lastdir, 6))[2, 4, 5];
   my $tm = stringtotime($lastdir); 
   my $age = strdifftime($tm, time);
   my $fullpath = $registry{basedir} . '/' . $directory;

   if ( -d $fullpath or $showdeleted) {
      $number++;
      if ( -d $fullpath) {
	 $totalsize = 0;
   	 find sub { $totalsize += -s }, $fullpath;
      }            
      if (open(STATUSFILE, "< $fullpath/$statusfile")) {
	 my $s = <STATUSFILE>;
	 chomp($s);
	 if ($s =~ /(\d+)\s(\d+)/) {
	    $goodfiles = $1;
	    $totalfiles = $2;
	 }
	 close(STATUSFILE);
      }

      if (not -d $fullpath) { 
	 $username = "-DELETED-"; 
      }
      printcolor(sprintf($body, basename($directory), $section, $age, $username, 
	 $totalfiles == 0 ? "--" : sprintf("%02d", $goodfiles), 
	 $totalfiles == 0 ? "--" : sprintf("%02d", $totalfiles), 
	 $totalsize / 0x100000));
   }   
}
printcolor($footer);

# --- end of file ---
