#!/usr/bin/perl -w

##############################################################################
#                                                                           #
#  mp3daily.pl V0.2 - Daily Dir and Link script (c) by FEJF 2002            #
#                                                                           #
#  creates daily dirs and links all rels of yesterday to artist/genre/group #
#  dirs while removing dead links. written for openftpd 0.30.x              #
#                                                                           #
#  fejf@gmx.de                                             www.openftpd.org #
#                                                                           #
# --------------------------------------------------------------------------#
#                                                                           #
#  This program is free software; you can redistribute it and/or modify     #
#  it under the terms of the GNU General Public License as published by     #
#  the Free Software Foundation; either version 2 of the License, or        #
#  (at your option) any later version.                                      #
#                                                                           #
#############################################################################

### TODO
#   - generate a .day-filled from all filled files from a day -> stats-generation

use strict;
use mp3info;
use primetools;

########################################
# set these vars to suit your needs ;) #
########################################

# be verbose
my $debug        = 1;

# set this to 1 for artist-dirs ignoring case (a* and A* go to the same dir)
my $ignorecase   = 1;

# set this to 1 if u want to parse the whole MP3 dir instead of only the today-dir
# e.g. if u allow backfilling u should set it to 1
my $parse_all    = 0;

# .permission file setup
my $permfile     = ".permissions";
my $perms        = pack "i2H8i9", 0, 0, "37377F68", 0, 0, 0, 0, 0, 0, 0, 0, 0;

# dirs where to link to...
my $todaydir     = "!today";
my $yesterdaydir = "!yesterday";
my $artistdir    = "/home/openftpd/site/MP3/!artist/";
my $genredir     = "/home/openftpd/site/MP3/!genre/";
my $groupdir     = "/home/openftpd/site/MP3/!group/";

# regexp for dirs u want to ignore (e.g. NUKED / INCOMPLETE)
my $ignoredirs   = "(NUKED|INCOMPLETE)";

# vars for announces
my $sitename     = "ABC";
my $botpidfile   = "/home/openftpd/etc/pid.eggdrop";
my $announcefile = "/home/openftpd/etc/announce.txt";
my $formatfile   = "/home/openftpd/msg/irc/newday";

# add the dirs where u want to check for mp3z...
my @dailydirs      = ( "/home/openftpd/site/MP3" );

# archive dirs - you need to set $parse_all to 1 to scan these dirs
my @archivedirs  = ( "/home/openftpd/site/MP3/!archiv" );

########################################

my ($file, %mp3info, $group, %groupcount, $dummy);
my ($relcount, $nukecount, $size, $format, @rules);
my ($day, $month, $year, $today, $yesterday, $daybefore);
my ($gdir, $grdir, $adir, $dir, $subdir, $reldir, @dirs, @daydirs, @top);


#get dates
($day, $month, $year) = (localtime())[3,4,5];
$month++;
$year += 1900;
$today = sprintf ("%04i-%02i-%02i",$year,$month,$day);
($day, $month, $year) = (localtime(time-86400))[3,4,5];
$month++;
$year += 1900;
$yesterday = sprintf ("%04i-%02i-%02i",$year,$month,$day);
($day, $month, $year) = (localtime(time-172800))[3,4,5];
$month++;
$year += 1900;
$daybefore = sprintf ("%04i-%02i-%02i",$year,$month,$day);
if ($debug) { print "(**) Date: DayBe4Yesterday: $daybefore, Yesterday: $yesterday and Today: $today\n"; }

# check dirs for broken links
if ($debug) { print "(**) checking for broken links\n"; }
opendir(DIR, $genredir) or die "(EE) can't open $genredir";
@dirs=readdir(DIR);
closedir(DIR);

# parse all subdirs and check for broken links
foreach $dir (@dirs) {
	if($dir =~ /^\.{1,2}/) {
		next;
	}
	if ($debug) { print "(II)  -> checking $dir\n"; }
	$subdir = $genredir.$dir;
	opendir (DIR, $subdir) or die "(EE) can't open $subdir";
	while($gdir = readdir(DIR)) {
		if($gdir =~ /^\.{1,2}/) {
			next;
		}
		if ($debug) { print "(II)     -> checking $gdir\n"; }
		if(!(-d "$subdir/$gdir")) {
			$gdir =~ /.+-(\w+)$/ and $grdir = "$groupdir$1/$gdir";
			if($gdir =~ /^va-.+$/i) {
				$adir = $artistdir."VA/$gdir";
			}
			else {
				$gdir =~ /^(\w).+$/ and $adir = "$artistdir$1/$gdir";
			}
			$gdir = "$subdir/$gdir";
			if ($debug) { print "(II)        -> removing $gdir, $grdir, $adir\n"; }
			unlink $gdir;
			unlink $grdir;
			unlink $adir;
		}
	}
	closedir (DIR);
}

push @daydirs, @dailydirs;
if ($parse_all) {
	push @daydirs, @archivedirs;
}

# create symlinks for !today and !yesterday
if ($debug) { print "(**) create new daily-symlinks\n"; }
foreach $dir (@dailydirs) {
	if ($debug) { print "(II) removing old daydir-links\n"; }
	unlink "$dir/$todaydir";
	unlink "$dir/$yesterdaydir";
	if ($debug) { print "(II) creating new daydir $dir/$today\n"; }
	mkdir "$dir/$today";
	open(PERMFILE,">$dir/$today/$permfile");
	print PERMFILE $perms;
	close(PERMFILE);
	symlink("$dir/$today","$dir/$todaydir") or print "(WW) can't create symlink from $dir/$today to $dir/$todaydir\n";
	symlink("$dir/$yesterday","$dir/$yesterdaydir") or print "(WW) can't create symlink from $dir/$yesterday to $dir/$yesterdaydir\n";
}

# parse new dirs
if ($debug) { print "(**) create new symlinks\n"; }
foreach $dir (@daydirs) {
	if($parse_all) {
		# use this if u allow backfilling
		if(!opendir (DIR, $dir)) {
			print "(WW) can't open $dir\n";
			@dirs="";
		}
		else {
			@dirs=readdir(DIR);
			closedir(DIR);
		}
	}
	else {
		# it's wise to parse yesterday and the day before yesterday if u don't close your today-dir @ 0:00...
		@dirs=( $yesterday, $daybefore );
	}
	foreach $subdir (@dirs) {
		if ($debug) { print "(II) parsing dir $subdir\n"; }
		if($subdir !~ /\d{4}-\d{2}-\d{2}/) {
			next;
		}
		opendir(DATEDIR, $dir."/".$subdir) or die "(EE) can't open $dir/$subdir";
		while($reldir=readdir(DATEDIR)) {
			if($reldir =~ /^\.{1,2}/) {
				next;
			}
			if($reldir =~ /$ignoredirs/) {
				if ($debug) { print "(II)  -> Skipping: $reldir\n"; }
				next;
			}
			opendir(RELDIR, $dir."/".$subdir."/".$reldir) or die "(EE) can't open $dir/$subdir/$reldir";
			if ($debug) { print "(II)  -> checking: $reldir\n"; }
			while($file=readdir(RELDIR)) {
				if (not defined $file) {
					next;
				}
				if ($file =~ /\.mp3$/) {
					last;
				}
			}
			if(not defined $file) {
				next;
			}
			closedir(RELDIR);
			%mp3info=mp3_info($dir."/".$subdir."/".$reldir."/".$file);
			$gdir = $genredir.$mp3info{GENRE};
			$gdir =~ s/\s/_/g;
			$reldir =~ /.+-(\w+)$/ and $grdir = $groupdir . $1;
			if($reldir =~ /^va-.+$/i) {
				$adir = $artistdir . "VA";
			}
			else {
				if($reldir =~ /^(\w).+$/) {
					if($ignorecase) {
						$adir = $artistdir . uc($1);
					}
					else {
						$adir = $artistdir . $1;
					}
				}
			}
			if ($debug) { print "(II)      -> $reldir gets linked to: $gdir, $grdir, $adir\n"; }
			if (!(-d $gdir)) {
				mkdir $gdir;
				open(PERMFILE,">$gdir/$permfile");
				print PERMFILE $perms;
				close(PERMFILE);
			}
			if (!(-d $grdir)) {
				mkdir $grdir;
				open(PERMFILE,">$grdir/$permfile");
				print PERMFILE $perms;
				close(PERMFILE);
			}
			if (!(-d $adir)) {
				mkdir $adir;
				open(PERMFILE,">$adir/$permfile");
				print PERMFILE $perms;
				close(PERMFILE);
			}
			if(!(symlink("$dir/$subdir/$reldir","$gdir/$reldir")) && $debug) { 
				print "(WW) can't create symlink to $gdir\n";
			}
			if(!(symlink("$dir/$subdir/$reldir","$grdir/$reldir")) && $debug) {
				print "(WW) can't create symlink to $grdir\n";
			}
			if(!(symlink("$dir/$subdir/$reldir","$adir/$reldir")) && $debug) {
				print "(WW) can't create symlink to $adir\n";
			}
		}
		closedir(DATEDIR);
	}
}

# finaly announce some stats :)
$relcount = 0;
$nukecount = 0;
$size = 0;
foreach $dir (@dailydirs) {
	if(!opendir (DIR, "$dir/$yesterday")) {
		if($debug) { print "(WW) can't open $dir/$yesterday - skipping for dircount\n"; }
		next;
	}
	while($subdir=readdir(DIR)) {
		if($subdir =~ /^\.{1,2}/) {
			next;
		}
		if($subdir =~ /NUKED/i) {
			$nukecount++;
		}
		else {
			$subdir =~ /-(\w+)$/ and $groupcount{$1}++;
			$relcount++;
		}
	}
	closedir (DIR);
	$subdir = `du -sk $dir/$yesterday`;
	$subdir =~ /^(\d+)\s/ and $size += $1;
}

$dummy=0;
foreach $group (sort {$groupcount{$b} <=> $groupcount{$a}} (keys %groupcount)) {
	if($debug) { print "(II) There are $groupcount{$group} releases from $group today...\n"; }
	$top[$dummy++]=$group;
}

open(FORMATFILE, "< $formatfile") or die "ERROR: $formatfile not found.\n";
$format = <FORMATFILE>;
close(FORMATFILE);
if (!$nukecount) {
	$nukecount = "NO";
}
if (!$relcount) {
	$relcount = "NO";
}
if (!$size) {
	$size = "ZERO ";
}
else {
	$size = int($size/1024);
}
@rules = (
	[ '$sitename'		, "$sitename"			],
	[ '$nukes'		, "$nukecount"			],
	[ '$rels'		, "$relcount"			],
	[ '$size'		, "$size"			],
	[ '$top1'		, "$top[0]"			],
	[ '$top2'		, "$top[1]"			],
	[ '$top3'		, "$top[2]"			],
	[ '$top4'		, "$top[3]"			],
	[ '$top5'		, "$top[4]"			],
	[ '$toprel1'		, "$groupcount{$top[0]}"	],
	[ '$toprel2'		, "$groupcount{$top[1]}"	],
	[ '$toprel3'		, "$groupcount{$top[2]}"	],
	[ '$toprel4'		, "$groupcount{$top[3]}"	],
	[ '$toprel5'		, "$groupcount{$top[4]}"	]
);
if ($announcefile & $botpidfile) {
	announce_line($announcefile, $botpidfile, substitute($format,0,@rules));
}
else {
	if ($debug) {
		print substitute($format,0,@rules);
	}
}
