#!/usr/bin/perl -w
use strict; #always
##############################################################################
#                                                                            #
#  primeCHECK v2.6 - FILE CHECKER (c) 2003 primemover / Team OpenFTPD        #
#                                                                            #
#  upload checker for OpenFTPD 0.29/0.30                                     #
#                                                                            #
# ---------------------------------------------------------------------------#
#                                                                            #
#  This program is free software; you can redistribute it and/or modify      #
#  it under the terms of the GNU General Public License as published by      #
#  the Free Software Foundation; either version 2 of the License, or         #
#  (at your option) any later version.                                       #
#                                                                            #
##############################################################################


my $standarddir = $ENV{'F4A_STD'};
if (not defined $standarddir) {
   print "ERROR: Unable to find FTP4ALL standard directory. \n",
         "Please set environment variable F4A_STD to this path\n";
   exit 1
}

# ----------------------------------------------------------------------------
#  enable / disable debug (verbose) output
# ----------------------------------------------------------------------------
my $debug = 0;

# ----------------------------------------------------------------------------
#  a list with all WANTED nfo files that should be extracted from zip's
# ----------------------------------------------------------------------------
my $nfofilelist = $standarddir."/etc/rel_nfo.lst";

# ----------------------------------------------------------------------------
#  these filetypes are checked against a sfv file (regular expression)
# ----------------------------------------------------------------------------
my $sfvtypes = '\.(?:rar|[rs0-9][0-9][0-9]|mp3|avi)';

# ----------------------------------------------------------------------------
#  these directories are no releases and should be handled in a special way
# ----------------------------------------------------------------------------
my $special_dirs = '(cd[0-9]|sample|disc[0-9])';


# ----------------------------------------------------------------------------
#  this suffix will be added for all missing files from a sfv check
#  set this to "" or 0 if you don't want to create tags for missings files
# ----------------------------------------------------------------------------
my $missingsuffix = "-missing";

# ----------------------------------------------------------------------------
#  rename all extracted nfo files to lower case ? (1 = yes / 0 = no)
# ----------------------------------------------------------------------------
my $lowercase_nfo = 1;

# ----------------------------------------------------------------------------
#  don't continue searching for nfos if one was found (1 = yes / 0 = no)
# ----------------------------------------------------------------------------
my $fastnfoscan   = 1;

# ----------------------------------------------------------------------------
#  show racestats as table (multiline output) (1 = yes / 0 = no)
#  if you like this thank twocom for forcing me to do this :)
#  and then thank topace for re-doing it with fmtfiles like everything else :)
# ----------------------------------------------------------------------------
my $race_tablestyle = 1;

# ----------------------------------------------------------------------------
#  show racestats during zipcheck (1 = yes / 0 = no)
# ----------------------------------------------------------------------------
my $race_stat_on_check = 1;

# ----------------------------------------------------------------------------
#  show id3tags during zipcheck (1 = yes / 0 = no)
# ----------------------------------------------------------------------------
my $id3_info_on_check = 1;

# ----------------------------------------------------------------------------
# type of status in releasedirectory (0 = statusfile / 1 = statusdirectory)
# ----------------------------------------------------------------------------
my $statustype = 0;

# ----------------------------------------------------------------------------
#  Do you want your users to be able of delete status dir? (1 = yes / 0 = no)
# ----------------------------------------------------------------------------
my $del_status = 1;

# ----------------------------------------------------------------------------
#  Do symlinks in parent dir and not status file?          (1 = yes / 0 = no)
#  If set to yes, it will override statusfilepath sections key.
#  You should change resolvesymlinks to yes in sections key too.
# ----------------------------------------------------------------------------
my $link_status = 0;

# ----------------------------------------------------------------------------
#  you don't have to change these settings usually
# ----------------------------------------------------------------------------

my $filledfile = ".filled";
my $unfilledfile = ".unfilled";
my $statusfile = ".status";

##############################################################################
# end of configuration (don't edit below unless you know what you are doing  #
##############################################################################

use Cwd;
use File::Basename;
use Fcntl;
use POSIX 'setsid';
use Bit::Vector::Overload;
Bit::Vector->Configuration("input=decimal");
Bit::Vector->Configuration("operators=arithmetic");

use FindBin;
use lib "$FindBin::Bin/";
use primetools;
use mp3info;
use imdb;
use aviinfo;

*STDERR = *STDOUT;

use vars qw($u $b);
my ($__filename,$message,$statusbar,$gf,$tf,%mp3info, $shutup,$tm1,
$__sitename,$__speed,$__username,$__groupname,$__pdir,$__announcefile,
$__botpidfile,$__path,$__adir,$__http_url,$raceinfo,%imdb,$__imdb_url,$username,$groupname,
%dircfg,%registry,$__permissionfile,$linkdir,@statusfiles,$totalfiles, $totalsize, 
$totalracers, $totalspeed,$linkpath,@racestats,$__crc, %aviinfo);

##############################################################################
# Format Section                                                             #
##############################################################################

format HEADER =
 !C.------------------------------------------------------------------------.!0
 !C|!3 !DOpenFTPD !HARCHIVE CHECKER                                               !0!C|!0
 !C|------------------------------------------------------------------------|!0
 !C| !c[!CFILE: !H@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< !c] !C|!0
                  $__filename
.

format MESSAGE =
 !C|!0 @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< !C|!0
$message
.

format FOOTER =
 !C`------------------------------------------------------------------------'!0
.
#`

format LIMITER =
 !C|------------------------------------------------------------------------|!0
.

format STATUS =
 !C| !H@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< [@>/@>]      !C|!0
       $statusbar,                                                $gf,$tf
.

format MP3INFO =
 !C| !c[!C ARTiST: !H@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< !c] [!CBiTRATE: !H@<<<<<<<<<<< !c] !C|!0
   $mp3info{'ARTIST'},                                $mp3info{'BITRATE'} . "kbps"
 !C| !c[!C  TiTLE: !H@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< !c] [!C   FREQ: !H@<<<<<<<<<<< !c] !C|!0
   $mp3info{'SONGNAME'},                              $mp3info{'FREQUENCY'} . "Hz"
 !C| !c[!C  ALBUM: !H@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< !c] [!C   MODE: !H@<<<<<<<<<<< !c] !C|!0
   $mp3info{'ALBUM'},                                 $mp3info{'MODE'}
 !C| !c[!C  GENRE: !H@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< !c] [!C   MPEG: !H@<<<<<<<<<<< !c] !C|!0
   $mp3info{'GENRE'},                                 $mp3info{'MPEG'}
 !C| !c[!C   YEAR: !H@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< !c] [!C  LAYER: !H@<<<<<<<<<<< !c] !C|!0
   $mp3info{'YEAR'},                                  $mp3info{'LAYER'}
 !C| !c[!CCOMMENT: !H@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< !c] [!C LENGTH: !H@<<<<<<<<<<< !c] !C|!0
   $mp3info{'COMMENT'},				      $mp3info{'LENGTH'}
.

# ----------------------------------------------------------------------------
#   get_formatfile ($event) 
# ----------------------------------------------------------------------------
sub get_formatfile {
	my $event = shift;
	open(FORMATFILE, "< $standarddir/msg/irc/$event") or return undef;
	my $formatstring = <FORMATFILE>;
	close(FORMATFILE);
	chomp($formatstring);
	return $formatstring;	
}

# ----------------------------------------------------------------------------
#   convertcomplete($instr)
#   converts the %C[___]* flags to what they should be
#   uses the same flag style as the who message.
# ----------------------------------------------------------------------------
sub convertcomplete {
	my $rs = "";
	my $string = shift;
	my @parts = split(/(%C\[(?:\w|:|-|%|\.)+\]\w)/, $string);
	foreach my $p (@parts) {
		if ($p =~ /(%C\[((?:\w|:|-|%|\.)+)\](\w))/) {
			# print "DEBUG: **$2** **$3**\n";
			$rs .=  sprintf($2, substitute($3, 1, @_));
		} else {
			$rs .= $p;
		}
	}
	return $rs;
}


# ----------------------------------------------------------------------------
#   extract_nfo($zipfile) - extract the group nfo file, ignore other nfos
# ----------------------------------------------------------------------------
sub extract_nfo {
	my $zipfile = $_[0];
	my $oldname;
	my $groupnfos;
	my @nfos = glob("*.nfo");
	if ($#nfos == -1 or $fastnfoscan == 0) {
		open(NFOLIST, "< $nfofilelist") or return 0;
		while (my $line = <NFOLIST>) {
			chomp($line);
			$line =~ s/^\s+//;
			$line =~ s/\s+$//;
			$groupnfos .= $line . " ";
		}
		close(NFOLIST);
		if ($groupnfos) {
			system("unzip -Coqq $zipfile $groupnfos $shutup");
		}
		if ($lowercase_nfo) {
			my @files = read_directory();
			foreach $oldname (@files) {
				if ($oldname =~ /\.(nfo|diz)$/i) {
					my $newname = lc($oldname);
					rename($oldname, $newname);
					if(($newname =~ /\.nfo$/i) and (exists $dircfg{'dccinfo'}) and ($dircfg{'dccinfo'} eq 'yes')) {
						system ('cp',$newname,$dircfg{dccpath}."/$__adir\_$newname");
						announce_dccinfo($newname);
					}
				}
			}
		}
	}
}


# ----------------------------------------------------------------------------
#   dizcount($array_with_whole_dizfile) - return number of total disks or 0
#   search for the number of disks in a file_id.diz file with syntax like
#   [01/03], [1/3], (01/03), [DiSK:o1/o3] ...
# ----------------------------------------------------------------------------
sub dizcount {
	my $total = 0;
	my @lines;
	if (-e "file_id.diz") {
		if (open(INPUT, "file_id.diz")) {
			while(my $line = <INPUT>) { 
				push(@lines, $line); 
			}
			close(INPUT);
		} else {
			print ("Unable to open file_id.diz\n");
		}
   	}
   	foreach my $line (@lines) {
		$line =~ s/[\r\n]//g;
		if ($line =~ m#[\[\(<:]\s?[\dox]{1,2}/([\dox]{1,2})[\]\)>]#i ||
		    $line =~ m# [\dox]{1,2}/([\dox]{1,2}) #i ||
		    $line =~ m#[^/]xx/(\d\d)$#i
		) {
		    $total = $1;
		    $total =~ s/o/0/;
		}
		if ($total != 0) {
		    last;
		}
	}
	if ($debug) { print "DEBUG: diz says $total files expected.\n"; }
	return $total;
}


# ----------------------------------------------------------------------------
#   write_filled($timestring, $username, $groupname, $path)
#   return 0 on error else 1, add a line to the filled file
#   if the an entry for the new file already exists it will be
#   overwritten.
# ----------------------------------------------------------------------------
sub write_filled {
	my $timestr   = shift;
	my $username  = shift;
	my $groupname = shift;
	my $path      = shift;
	my $speed     = shift;
	my @oldfile   = ();
	my @newfile   = ();
	
	sysopen(FILLEDFILE, $filledfile, O_RDWR|O_CREAT) or return 0;
	# LOCK_EX = 2
	flock(FILLEDFILE, 2) or return 0;
	
	while (<FILLEDFILE>) { push(@oldfile, $_); }
	
	foreach my $line (@oldfile) {
		chomp($line);
		if ((split(/\s+/, $line, 7))[6] !~ /\Q$path\E/i) {
			push(@newfile, $line);
		}
	}
	
	truncate (FILLEDFILE, 0) or return 0;
	# SEEK_SET = 0
	seek (FILLEDFILE, 0, 0) or return 0;
	
	foreach my $line (@newfile) { 
		print FILLEDFILE "$line\n"; 
	}
	printf FILLEDFILE "%s %-10s %-10s %10d %10d %s\n", $timestr, 
		$username, $groupname, -s $path, $speed, $path;
	# LOCK_UN = 8
	flock(FILLEDFILE, 8);
	close(FILLEDFILE);
	return 1;
}

# ----------------------------------------------------------------------------
#   write_unfilled($crc, $timestring, $username, $groupname, $path)
#   return 0 on error else 1, add a line to the .unfilled file
#   if an entry for the new file already exists it will be overwritten.
#   this is nearly the same like write_filled - except that it stores
#   also the crc of an uploaded file for later "site check"
# ----------------------------------------------------------------------------
sub write_unfilled {
	my $crcsum    = shift;
	my $timestr   = shift;
	my $username  = shift;
	my $groupname = shift;
	my $path      = shift;
	my $speed     = shift;
	my $line;
	my @oldfile   = ();
	my @newfile   = ();
	
	sysopen(UNFILLEDFILE, $unfilledfile, O_RDWR|O_CREAT) or return 0;
	# LOCK_EX = 2
	flock(UNFILLEDFILE, 2) or return 0;
	
	while (<UNFILLEDFILE>) { push(@oldfile, $_); }
	
	foreach $line (@oldfile) {
		chomp($line);
		if ((split(/\s+/, $line, 8))[7] !~ /\Q$path\E/i) {
			push(@newfile, $line);
		}
	}
	
	truncate (UNFILLEDFILE, 0) or return 0;
	# SEEK_SET = 0
	seek (UNFILLEDFILE, 0, 0) or return 0;
	
	foreach $line (@newfile) { 
		print UNFILLEDFILE "$line\n"; 
	}
	printf UNFILLEDFILE "%s %s %-10s %-10s %10d %10d %s\n", $crcsum, $timestr, 
		$username, $groupname, -s $path, $speed, $path;
	# LOCK_UN = 8
	flock(UNFILLEDFILE, 8);
	close(UNFILLEDFILE);
	return 1;
}



# ----------------------------------------------------------------------------
#   statusbar($total, $good, $sizeofbar, $style)
#   create a string with a status bar with the percent of complete files
# ----------------------------------------------------------------------------
sub statusbar {
	my ($factor, $percent);
	my ($good, $bad, $bar1, $bar2);

	if (!$_[0]) {
		return("STATUSBAR not available");
	}
	if ($_[0] == 0) { 
		$factor = 1; $percent = 0; 
	} 
	else {
		$factor = $_[2] / $_[0];
		$percent = $_[1] / $_[0] * 100;
	}
	$good = $_[1] * $factor;
	$bad = ($_[0] - $_[1]) * $factor;
	$bar1 = sprintf("-%s-[%d of %d files]-[%02.0f%%]-[%s%s]-", 
	                   $__sitename, $_[1], $_[0], $percent, '#' x $good, 
	                   '-' x $bad);

	$bar2 = sprintf("%02.0f%% [%s%s]", $percent, '#' x $good, 
	                   ':' x $bad);
	$_[3] == 1 ? return $bar2 : return $bar1;
}


# ----------------------------------------------------------------------------
#   create_missing(@filenames)
#   create for all filenames in the array a $filename$missingsuffix file
# ----------------------------------------------------------------------------
sub create_missing {
   my $filename;

format MISSING =
 !C| !b[MiSSiNG]!H @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< !C|!0
	           $filename
.

	if ($missingsuffix) {
		printf " !C|" . '-' x 72 . "!C|!0\n";
		foreach $filename (sort @_) {
			$filename = lc($filename);
			if (-e $filename) {
				next;
			}
			$~ = "MISSING"; write;
			open(MISSINGFILE, "> ${filename}$missingsuffix");
			close(MISSINGFILE);
		}
	}
}

# ----------------------------------------------------------------------------
#   announce_badfile()
#   announce the filename of any bad uploaded file in configured sections
# ----------------------------------------------------------------------------
sub announce_badfile {
	my ($badfileformat, @rules);

	$badfileformat = get_formatfile("badfile");

	@rules = (
		[ '$sitename'   , "$__sitename"           ],
		[ '$filename'   , "$__filename"           ],
		[ '$username'   , "$__username"           ],
		[ '$groupname'  , "$__groupname"          ],
		[ '$parentdir'  , "$__pdir"               ],
		[ '$directory'  , "$__adir"               ]
	);
	if ($__announcefile and $__botpidfile) {
		announce_line($__announcefile, $__botpidfile, substitute($badfileformat, 0, @rules));
	}
}

# ----------------------------------------------------------------------------
#   announce_dccinfo($filename)
#   announce info of how to get a nfo-file via dcc from the sitebot
# ----------------------------------------------------------------------------
sub announce_dccinfo($) {
	my ($filename, $fileformat, @rules);

	$filename = shift;
	$fileformat = get_formatfile("dccinfo");
	@rules = (
		[ '$sitename'   , "$__sitename"           ],
		[ '$filename'   , "$filename"             ],
		[ '$username'   , "$__username"           ],
		[ '$groupname'  , "$__groupname"          ],
		[ '$parentdir'  , "$__pdir"               ],
		[ '$directory'  , "$__adir"               ]
	);
	if ($__announcefile and $__botpidfile) {
		announce_line($__announcefile, $__botpidfile, substitute($fileformat, 0, @rules));
	}
}                                                          

# ----------------------------------------------------------------------------
#   announce_http()
#   announce a http url (e.g. from a iso-release)
# ----------------------------------------------------------------------------
sub announce_http {
	my ($fileformat, @rules);

	$fileformat = get_formatfile("http");
	@rules = (
		[ '$sitename'   , "$__sitename"           ],
		[ '$filename'   , "$__filename"           ],
		[ '$username'   , "$__username"           ],
		[ '$groupname'  , "$__groupname"          ],
		[ '$parentdir'  , "$__pdir"               ],
		[ '$directory'  , "$__adir"               ],
		[ '$url'        , "$__http_url"           ]
	);
	if ($__announcefile and $__botpidfile) {
		announce_line($__announcefile, $__botpidfile, substitute($fileformat, 0, @rules));
	}
}

# ----------------------------------------------------------------------------
#   announce_id3info($totalfiles)
#   update announce after the first file was uploaded
# ----------------------------------------------------------------------------
sub announce_id3info {
	my $totalfiles = $_[0];
	my $ircformat = get_formatfile("id3info");
	my $updatespeed = sprintf("%.1f", $__speed / 1024.0);
	

	(my $minutes, my $hours) = (localtime)[1..2];
	my $time = sprintf("%02d:%02d", $hours, $minutes);

	if (not %mp3info) { %mp3info = mp3_info($__filename); }

	my @rules = (
		[ '$artist'     , "$mp3info{ARTIST}"      ],
		[ '$album'      , "$mp3info{ALBUM}"       ],
		[ '$genre'      , "$mp3info{GENRE}"       ],
		[ '$year'       , "$mp3info{YEAR}"        ],
		[ '$bitrate'    , "$mp3info{BITRATE}kbps" ],
		[ '$mode'       , "$mp3info{MODE}"        ],
		[ '$frequency'  , "$mp3info{FREQUENCY}Hz" ],
		[ '$sitename'   , "$__sitename"           ],
		[ '$totalfiles' , "$totalfiles"           ],
		[ '$username'   , "$__username"           ],
		[ '$groupname'  , "$__groupname"          ],
		[ '$parentdir'  , "$__pdir"               ],
		[ '$speed'      , "$updatespeed"          ],
		[ '$directory'  , "$__adir"               ]
	);

	if ($__announcefile and $__botpidfile and $ircformat) {
		announce_line($__announcefile, $__botpidfile, substitute($ircformat, 0, @rules));
	}
}


# ----------------------------------------------------------------------------
#   announce_update($totalfiles)
#   update announce after the first file was uploaded
# ----------------------------------------------------------------------------
sub announce_update {
	my $totalfiles = shift;
	my $filesize = filesize_to_string(-s $__path);
	my $estimated = filesize_to_string((-s $__path) * $totalfiles);
	my $updatespeed = sprintf("%.1f", $__speed / 1024.0);
	my $ircformat = get_formatfile("update");
	
	(my $minutes, my $hours) = (localtime)[1..2];
	my $time = sprintf("%02d:%02d", $hours, $minutes);

	my $ratio="unknown";

	if($__filename =~ /\.zip/i) {
		$ratio="zip";
	}
	if($__filename =~ /\.(rar|[rs0-9][0-9][0-9])/i) {
		open(RARFILE, "$__filename") or die " can not open $__filename for ratio check";
		if(!read RARFILE, $ratio, 46) {
			printf("ERROR while trying to get rar-ratio !\n");
		}
		else {
# get only the 46th byte (= compression ratio ;)
			($ratio)=($ratio =~ /(.)$/);
			if($ratio =~ /0/) {
				$ratio = "none";
			}
			else {
				$ratio = "m".$ratio;
			}
		}
		close(RARFILE);
	}
	
	my @rules = (
		[ '$sitename'   , "$__sitename"           ],
		[ '$totalfiles' , "$totalfiles"           ],
		[ '$username'   , "$__username"           ],
		[ '$groupname'  , "$__groupname"          ],
		[ '$parentdir'  , "$__pdir"               ],
		[ '$directory'  , "$__adir"               ],
		[ '$filesize'   , "$filesize"             ],
		[ '$estimated'  , "$estimated"            ],
		[ '$time'       , "$time"                 ],
		[ '$ratio'      , "$ratio"                ],
		[ '$speed'      , "$updatespeed"          ]
	);
	
	if ($__announcefile and $__botpidfile and $ircformat) {
		announce_line($__announcefile, $__botpidfile, substitute($ircformat, 0, @rules));
	}
}


# ----------------------------------------------------------------------------
#   announce_complete($totalfiles)
#   complete announce after the last file was uploaded
# ----------------------------------------------------------------------------
sub announce_complete {
	my $ircformat1 = get_formatfile('complete');
	my $ircformat2 = get_formatfile('race');
	my $announcemsg = "";

   	@racestats = calc_racestats();
        ($totalfiles, $totalsize, $totalracers, $totalspeed) = 
        split(/:/, $racestats[0], 4);
	$totalsize = (split /:/, $racestats[0])[1];
   
   	$totalspeed = $totalspeed / 1024.0;
   	$totalsize = filesize_to_string($totalsize);
   	my $totaltime = calc_totaltime();
   	shift(@racestats);

	# racestats in tableformat
	# use table style if there are more than one racers
	if (($totalracers > 1) and ($race_tablestyle == 1)) {
		my $fmtfile_completetable_top  = $standarddir."msg/irc/completetable_top";
		my $fmtfile_completetable_body = $standarddir."msg/irc/completetable_body";
		my $fmtfile_completetable_tail = $standarddir."msg/irc/completetable_tail";

		# Open the top of the complete table, and do each line in it
		open (COMTOP, "< $fmtfile_completetable_top");
		my @rules = (
			[ "N", "$__sitename"  ],
			[ "Y", "$__pdir"      ],
			[ "D", "$__adir"      ],
			[ "T", "$totaltime"   ],
			[ "I", "$totalsize"   ],
			[ "A", "$totalracers" ]
		);
		while (<COMTOP>) {
			$announcemsg .= convertcomplete(convertcolors($_), @rules);
		}
		close (COMTOP);

		foreach my $line (@racestats) {
			# Split the race stats into each variable
			my ($rank, $username, $groupname, $files, $size, 
			    $speed, $percent) = split(/:/, $line, 7);
			$size = filesize_to_string($size);
			$speed = $speed / 1024.0;

			my @rules2 = (
				[ "R", "$rank"        ],
				[ "U", "$username"    ],
				[ "G", "$groupname"   ],
				[ "F", "$files"       ],
				[ "S", "$size"        ],
				[ "P", "$speed"       ],
				[ "C", "$percent"     ],
			);
			# Open the body file, and do each line
			open (COMBODY, "< $fmtfile_completetable_body");
			while (<COMBODY>) {
				$announcemsg .= convertcomplete(convertcolors($_), @rules2);
			}
			close (COMBODY);
		}

		# Open the tail file and do each line
		open (COMTAIL, "< $fmtfile_completetable_tail");
		while (<COMTAIL>) {
			$announcemsg .= convertcomplete(convertcolors($_), @rules);
		}
		close (COMTAIL);
	}
	# racestats on a single line
	else {
		foreach my $line (@racestats) {
			my ($rank, $username, $groupname, $files, $size, 
			    $speed, $percent) = split(/:/, $line, 7);
			
			$size = filesize_to_string($size);
			$speed = sprintf("%.1f", $speed / 1024.0);
			my @rules = (
				[ '$rank'       , "$rank"      ],
				[ '$username'   , "$username"  ],
				[ '$groupname'  , "$groupname" ],
				[ '$files'      , "$files"     ],
				[ '$size'       , "$size"      ],
				[ '$percent'    , "$percent"   ],
				[ '$speed'      , "$speed"     ]
			);
         		$raceinfo .= substitute($ircformat2, 0, @rules) . ' ';
      		}
		my @rules = (
			[ '$sitename'   , "$__sitename"           ],
			[ '$username'   , "$__username"           ],
			[ '$groupname'  , "$__groupname"          ],
			[ '$parentdir'  , "$__pdir"               ],
			[ '$directory'  , "$__adir"               ],
			[ '$totalfiles' , "$totalfiles"           ],
			[ '$totalsize'  , "$totalsize"            ],
			[ '$totaltime'  , "$totaltime"            ],
			[ '$raceinfo'   , "$raceinfo"             ]
		);

		if ($ircformat1) {
			$announcemsg = substitute($ircformat1, 0, @rules);
		}		
	}
	if ($__announcefile and $__botpidfile and $ircformat1) {
		announce_line($__announcefile, $__botpidfile, 
		              $announcemsg);
	}
}


# ----------------------------------------------------------------------------
#   announce_joinrace()
#   joinrace announce if a new user started uploading files of the release
# ----------------------------------------------------------------------------
sub announce_joinrace {
	my $ircformat = get_formatfile('joinrace');
	my $joinspeed = sprintf("%.1f", $__speed / 1024.0);
		
	my @rules = (
		[ '$sitename'   , "$__sitename"           ],
		[ '$username'   , "$__username"           ],
		[ '$groupname'  , "$__groupname"          ],
		[ '$directory'  , "$__adir"               ],
		[ '$speed'      , "$joinspeed"            ]
	);

	if ($__announcefile and $__botpidfile and $ircformat) {
		announce_line($__announcefile, $__botpidfile, substitute($ircformat, 0, @rules));
	}
}


# ----------------------------------------------------------------------------
#   announce_imdb()
#   announce info got from imdb.com
# ----------------------------------------------------------------------------
sub announce_imdb {
	my $ircformat = get_formatfile('imdb');
	if (!($imdb{plot})) {$imdb{plot}="n/a";}
	if (!($imdb{genre})) {$imdb{genre}="n/a";}
	if (!($imdb{tagline})) {$imdb{tagline}="n/a";}
	if (!($imdb{rating})) {$imdb{rating}="?";}
	if (!($imdb{votes})) {$imdb{votes}="no";}
	my @rules = (
		[ '$sitename'   , "$__sitename"           ],
		[ '$username'   , "$__username"           ],
		[ '$groupname'  , "$__groupname"          ],
		[ '$directory'  , "$__adir"               ],
		[ '$parentdir'  , "$__pdir"               ],
		[ '$title'      , "$imdb{title}"          ],
                [ '$rating'     , "$imdb{rating}"         ],
                [ '$votes'      , "$imdb{votes}"          ],
                [ '$plot'       , "$imdb{plot}"           ],
		[ '$genre'      , "$imdb{genre}"          ],
                [ '$tagline'    , "$imdb{tagline}"        ],
		[ '$url'        , "$__imdb_url"           ]
	);
	if ($__announcefile and $__botpidfile and $ircformat) {
		announce_line($__announcefile, $__botpidfile, substitute($ircformat, 0, @rules));
	}
}

# ----------------------------------------------------------------------------
#   announce_sample()
#   announce info got from an avi in a dir called sample ;)
# ----------------------------------------------------------------------------
sub announce_sample {
	my $ircformat = get_formatfile('sample');

	if (not %aviinfo) { %aviinfo = aviinfo($__filename); }

	my @rules =(
		[ '$sitename'   , "$__sitename"           ],
		[ '$username'   , "$__username"           ],
		[ '$groupname'  , "$__groupname"          ],
		[ '$directory'  , "$__adir"               ],
		[ '$parentdir'  , "$__pdir"               ],
		[ '$res'	, "$aviinfo{width}x$aviinfo{height}" ],
		[ '$frames'	, "$aviinfo{fps}"	  ],
		[ '$codec'	, "$aviinfo{v_codec}" 	  ]
	);
	if ($__announcefile and $__botpidfile and $ircformat) {
		announce_line($__announcefile, $__botpidfile, substitute($ircformat, 0, @rules));
	}
}

# ----------------------------------------------------------------------------
#   announce_status25()
#   announce a race halfway info
# ----------------------------------------------------------------------------
sub announce_status25 {
	my ($fileformat, @rules);

	$fileformat = get_formatfile("status25");
	@rules = (
		[ '$sitename'   , "$__sitename"           ],
		[ '$username'   , "$__username"           ],
		[ '$groupname'  , "$__groupname"          ],
		[ '$parentdir'  , "$__pdir"               ],
		[ '$directory'  , "$__adir"               ],
	);
	if ($__announcefile and $__botpidfile) {
		announce_line($__announcefile, $__botpidfile, substitute($fileformat, 0, @rules));
	}
}

# ----------------------------------------------------------------------------
#   announce_status50()
#   announce a race halfway info
# ----------------------------------------------------------------------------
sub announce_status50 {
	my ($fileformat, @rules);

	$fileformat = get_formatfile("status50");
	@rules = (
		[ '$sitename'   , "$__sitename"           ],
		[ '$username'   , "$__username"           ],
		[ '$groupname'  , "$__groupname"          ],
		[ '$parentdir'  , "$__pdir"               ],
		[ '$directory'  , "$__adir"               ],
	);
	if ($__announcefile and $__botpidfile) {
		announce_line($__announcefile, $__botpidfile, substitute($fileformat, 0, @rules));
	}
}

# ----------------------------------------------------------------------------
#   announce_status75()
#   announce a race halfway info
# ----------------------------------------------------------------------------
sub announce_status75 {
	my ($fileformat, @rules);

	$fileformat = get_formatfile("status75");
	@rules = (
		[ '$sitename'   , "$__sitename"           ],
		[ '$username'   , "$__username"           ],
		[ '$groupname'  , "$__groupname"          ],
		[ '$parentdir'  , "$__pdir"               ],
		[ '$directory'  , "$__adir"               ],
	);
	if ($__announcefile and $__botpidfile) {
		announce_line($__announcefile, $__botpidfile, substitute($fileformat, 0, @rules));
	}
}

# ----------------------------------------------------------------------------
#   calc_racestats() - return list with all race info
# ----------------------------------------------------------------------------
sub calc_racestats {
	my %racestats   = ();   
	my $totalfiles  = 0;   
	my $totalsize   = Bit::Vector->new_Dec(64, "0");
	my $totalracers = 0;
	my $totalspeed  = 0;
        my $percent     = 0.0;
        my @result      = ();

	open(FILLEDFILE, "< $filledfile") or return 0;
	while (<FILLEDFILE>) {
		chomp($_);
		my ($username, $groupname, $size, $speed, $filename) = 
		(split(/\s+/, $_, 7))[2..6];
      
		if (-f $filename) {
			if (not exists $racestats{$username}) {
				$racestats{$username} = 
				[ $groupname, 1, undef, $speed ];
				$racestats{$username}[2] = Bit::Vector->new_Dec(64, $size);
			} else {
				$racestats{$username}[1] ++;
				$racestats{$username}[2] += $size;
				$racestats{$username}[3] += $speed;
			}
		}
	}
	close(FILLEDFILE);

	# calculate totals
	foreach my $racer (keys %racestats) {
		$totalfiles += $racestats{"$racer"}[1];
		$totalsize  += $racestats{"$racer"}[2];
		$totalspeed += $racestats{"$racer"}[3];
		$totalracers ++;
	}
	$totalspeed = sprintf("%.1f", $totalfiles ? $totalspeed / $totalfiles : 0);

	# store total values in the first line
	push(@result, sprintf("%d:%s:%d:%d", $totalfiles, $totalsize->to_Dec, $totalracers,
		$totalspeed));

	# store the sorted racestats in a list with line format:
	# rank:username:groupname:files:size:speed:percent
	my $i = 1;
	foreach my $racer (sort {$racestats{$b}[2]->to_Dec <=> $racestats{$a}[2]->to_Dec} keys %racestats) {
		if ($totalsize != 0) {
		    $percent = sprintf("%.1f", $racestats{$racer}[2]->to_Dec
				       / $totalsize->to_Dec * 100);
		}
		push(@result, sprintf("%d:%s:%s:%d:%s:%d:%.1f", $i, $racer,
				      $racestats{$racer}[0], $racestats{$racer}[1],
				      $racestats{$racer}[2]->to_Dec, $racestats{$racer}[3] /
				      $racestats{$racer}[1], $percent
				      )
		     );
      		$i++;
	}
	return @result;
}


# ----------------------------------------------------------------------------
#   count_good() - return number of checked files
# ----------------------------------------------------------------------------
sub count_good {
	my $counter = 0;
	open(FILLEDFILE, "< $filledfile") or return 0;
	while (<FILLEDFILE>) {
		my $directory = (split(/\s+/, $_, 7))[6];
		chomp($directory);
		if (-f $directory) {
			$counter++;
		}
	}
	close(FILLEDFILE);
	return $counter;
}


# ----------------------------------------------------------------------------
#   status($goodfiles, $totalfiles)
#   check the status of the upload and announce if the first or last file
#   was uploaded
# ----------------------------------------------------------------------------
sub status {
	my ($rank, $usr, $grp, $files, $size, $speed, $percent);
        my ($totalfiles, $totalsize, $totalracers, $totalspeed);

# ----------------------------------------------------------------------------

format RACE_TOP =
 !C|------------------------------------------------------------------------|!0
 !C|!c RK | Racer      | Group      |Files|  Totalsize |       Speed |Percent !C|!0
 !C|!c----+------------+------------+-----+------------+-------------+--------!C|!0
.

format RACE_BODY =
 !C|!H @> !c| !H@<<<<<<<<< !c|!H @<<<<<<<<< !c|!H @>> !c|!H @>>>>>>>>> !c|!H @>>>>> kb/s !c|!H @>>>>% !C|!0
       $rank,   $username,       $groupname,      $files,   $size,           $speed / 1024.0,     $percent
.
# ----------------------------------------------------------------------------

	$gf = $_[0];
	$tf = $_[1];

	$tf =~ s/^0+//; # remove leading zeros

	my ($perm,$statusfilepath);
	if ($del_status) {
		$perm = pack "i12", 0, 0, "723723", 0, 0, 0, 0, 0, 0, 0, 0, 0;
	} else {
		$perm = pack "i12", 0, 0, "197379", 0, 0, 0, 0, 0, 0, 0, 0, 0;
	}

	$statusfilepath = "..";
	if (exists $dircfg{statusfilepath}) {
		if (substr($dircfg{statusfilepath}, 0, 1) eq '/') { # absolute path
	    	    $statusfilepath = $registry{basedir} . $dircfg{statusfilepath};
		} else {
		    $statusfilepath = $dircfg{statusfilepath};
		}
	}

	if ($link_status) {
		$statusfilepath = "..";
	}

	# clean up
	my @files = read_directory();
	foreach my $fn (@files) {
		if ($fn =~ /-\[.+/) {
			unlink($fn . "/" . $__permissionfile);

			if ($statustype == 0) { 
			    unlink($fn);
			} 
			elsif ($statustype == 1) { 
			    rmdir($fn);
			}
		}
		if ($fn =~ /\Q${__filename}${missingsuffix}\E/i) {
		    unlink $fn;
		}
	}
	if (-e $__filename . $missingsuffix) {
		unlink $__filename . $missingsuffix;
	}

	if ($link_status) {
		if (opendir(DIR, $statusfilepath . $linkdir)) {
			$__adir =~ s/\s/_/;
			@statusfiles = grep {/-\[.+\]-\[.+\]-\[\Q$__adir\E\]-/} readdir(DIR);
			closedir(DIR);
			foreach my $sf (@statusfiles) {
				unlink($statusfilepath . $linkdir . "/" . $sf);
	     		}
	  	}
	} else {
		if (opendir(DIR, $statusfilepath)) {
			$__adir =~ s/\s/_/;
			@statusfiles = grep {/-\[.+\]-\[.+\]-\[\Q$__adir\E\]-/} readdir(DIR);
			closedir(DIR);
			foreach my $sf (@statusfiles) {
				unlink($statusfilepath . "/" . $sf);
			}
		}
	}

	##########################
	# Calculate Racestats    #
	##########################
	if ($gf > 0) {
		@racestats = calc_racestats();
		($totalfiles, $totalsize, $totalracers, $totalspeed) = split(/:/, $racestats[0], 4);
		$totalspeed = $totalspeed / 1024;
		shift(@racestats);
		my $totaltime = calc_totaltime();
		$totalsize = filesize_to_string($totalsize);
		if ($debug) { print "DEBUG: TF: $totalfiles, TS: $totalsize, TR: $totalracers, SPEED: $totalspeed\n"; }

		# Stats during Race
		if ($race_stat_on_check) {
			$~ = "RACE_TOP"; write;
			foreach my $line (@racestats) {
				# Split the race stats into each variable
				($rank, $username, $groupname, $files, $size, $speed, $percent) = 
				split(/:/, $line, 7);
				$size = filesize_to_string($size);
				$~ = "RACE_BODY"; write;
			}
		}
                $~ = "LIMITER";write;
                $statusbar = statusbar($tf, $gf, 50, 1);
                $~ = "STATUS";write;
	}

	# create the hidden statusfile for script usage
	if (open(STATUSFILE, "> $statusfile")) {
		print STATUSFILE "$gf $tf\n";
		close(STATUSFILE);
	}

	######################################
	# release is complete                #
	######################################
	if (($tf) and ($gf == $tf) and ($gf > 0)) {
		if ($tf == 1 and $dircfg{'announce_update'} eq 'yes' and $__filename =~ /\.mp3$/i) {
			# if there's only 1 mp3 and rel is already finished do announce_update to post genre'n stuff to chan
			announce_id3info($tf);
		}
		if (exists $dircfg{complete_dir} and $dircfg{complete_dir} ne "") {
			create_complete($tf, $totalsize);
		}
		calc_totaltime();
		# copy site nfo to directory
		if (exists $dircfg{siteinfofile}) {
			my $sitenfofile = $dircfg{siteinfofile};
			if ( -e $sitenfofile and exists $dircfg{copyinfofile}
				and $dircfg{copyinfofile} eq 'yes') {
					system ('cp',$sitenfofile,'.');
		 	}
		}
		# lock directory
		if (exists $dircfg{lock_complete} and $dircfg{lock_complete} eq "yes") {
			open(LOCKFILE, "> .locked");
			close(LOCKFILE);
		}

		if (exists $dircfg{'announce_complete'} and $dircfg{'announce_complete'} eq 'yes') {
			announce_complete();
		}
		
		#poing m3u generation
		if (exists $dircfg{'generate_m3u'} and $dircfg{'generate_m3u'} eq 'yes') {
			$message = "!HGenerating M3U based on received filenames...";
			$~ = "MESSAGE"; write;
			generate_m3u();
		}
	}

	###############################
	# release is not complete     #
	###############################
	if (($tf) and ($gf < $tf)) {
		my $status = statusbar($tf, $gf, 10, 0);
		create ($status);
		
		if (exists $dircfg{'announce_status25'} and $dircfg{'announce_status25'} eq 'yes') {
  		  my $mod = $tf - (int($tf/2) * 2);
		  if ($tf  > 40) {
		    if ($mod eq 1) { if ($gf eq (int($tf/4) + 1)) { announce_status25(); }
		            } else { if ($gf eq int($tf/4)) { announce_status25(); }
                    }
		  }
		}

		if (exists $dircfg{'announce_status50'} and $dircfg{'announce_status50'} eq 'yes') {
  		  my $mod = $tf - (int($tf/2) * 2);
		  if ($tf  > 10) {
		    if ($mod eq 1) { if ($gf eq (int($tf/2) + 1)) { announce_status50(); }
		            } else { if ($gf eq int($tf/2)) { announce_status50(); }
                    }
		  }
		}

		if (exists $dircfg{'announce_status75'} and $dircfg{'announce_status75'} eq 'yes') {
  		  my $mod = $tf - (int($tf/2) * 2);
		  if ($tf  > 40) {
		    if ($mod eq 1) { if ($gf eq (int($tf/2) + 1)) { announce_status75(); }
		            } else { if ($gf eq int($tf/2)) { announce_status75(); }
                    }
		  }
		}		
		
		
		my $statusfile = sprintf("/-%s-[%d of %d files]-[%.0f%%]-[%s]-",
				          $__sitename, $gf, $tf, $gf / $tf * 100,
				          $__adir);
		if ($link_status) {
			symlink($linkpath, $statusfilepath.$linkdir.$statusfile);
		} else {
			open(SF, "> $statusfilepath$statusfile" || printf "ERROR: Cant write statusfile\n");
			close(SF);
		}
		# update announce on first file if release has more then 1 disk
		if (($gf == 1) and (exists $dircfg{'announce_update'})) {
			if($dircfg{'announce_update'} eq 'yes') {
				if ($__filename =~ /.mp3$/i) {
					announce_id3info($tf);
				}
				elsif ($tf > 1) {
					announce_update($tf);
				}
			}
		}
	}
}


# ----------------------------------------------------------------------------
#   generate_m3u() - generates m3u file
# ----------------------------------------------------------------------------
sub generate_m3u {
	my @mp3list = sort glob("*.mp3");
	if ($mp3list[0]) {
	    my @sfv_name = glob("*.sfv");
	    $sfv_name[0] =~ s/\.sfv/\.m3u/ig;	
	    open M3U, ">$sfv_name[0]";
	    foreach my $mp3file (@mp3list) {
		print M3U $mp3file."\r\n";
	    }
	    close M3U;
	}
}


# ----------------------------------------------------------------------------
#   read_sfvfile($filename) - read a sfvfile to a hash
# ----------------------------------------------------------------------------
sub read_sfvfile {
	my $sfvfile = $_[0];
	my %sfvhash = ();
	local *SFVFILE;
	if (-s $sfvfile) {
	    open(SFVFILE, "< $sfvfile") or return %sfvhash;
	    while (<SFVFILE>) {
		if (($_ !~ /^;/) and ($_ =~ /(.+$sfvtypes) (\w{8})/i)) {
		    my $crc = $2;
		    # print "$file ==> $crc\n";
		    my $filename = lc($1);
		    $sfvhash{$filename} = $crc;
		}
	    }
	    close(SFVFILE);
	}
    return %sfvhash;
}


# ----------------------------------------------------------------------------
#   is_new_racer() - return 1 if the username was not found in .filled
# ----------------------------------------------------------------------------
sub is_new_racer {
	open(FILLEDFILE, "< $filledfile") or return 0;
	while (<FILLEDFILE>) {
		$username = (split(/\s+/, $_, 6))[2];
		if ($username eq $__username) {
			close(FILLEDFILE);
			return 0;
		}
	}
	close(FILLEDFILE);
	return 1;
}


# ----------------------------------------------------------------------------
#  strdifftime($earlier, $later) - calc the diff between two times as string
# ----------------------------------------------------------------------------
sub strdifftime {
	my $time1 = $_[0]; 
	my $time2 = $_[1];
	my $diffstr = "";  
	my $diff = $time2 - $time1;
	
	my $seconds = $diff % 60; $diff = ($diff - $seconds) / 60;
	my $minutes = $diff % 60; $diff = ($diff - $minutes) / 60;
	my $hours   = $diff % 24; $diff = ($diff - $hours) / 24;
	my $days    = $diff % 7;
	my $weeks   = ($diff - $days) / 7;
	
	if ($weeks != 0) {
		$weeks == 1 ? ($diffstr .= ($weeks . " week")) : 
			($diffstr .= ($weeks . " weeks"));
		if ($days or $hours or $minutes or $seconds) {
	 		$diffstr .= " ";
		}
	}

	if ($days != 0) {
		$days == 1 ? ($diffstr .= ($days . " day")) : 
			($diffstr .= ($days . " days"));
		if ($hours or $minutes or $seconds) {
	 		$diffstr .= " ";
		}
	}

	if ($hours != 0 and $weeks == 0) {
		$hours == 1 ? ($diffstr .= ($hours . " hour")) : 
			($diffstr .= ($hours . " hours"));
		if ($minutes or $seconds) {
	 		$diffstr .= " ";
		}
	}
	
	if ($minutes != 0 and $days == 0 and $weeks == 0) {
		$diffstr .= ($minutes . " min");
		if ($seconds) {
	 		$diffstr .= " ";
		}
	}
	if ($seconds != 0 and $hours == 0 and $days == 0 and $weeks == 0) {
		$diffstr .= ($seconds . " sec");
	}
	return $diffstr;
}


# ----------------------------------------------------------------------------
#  calc_totaltime() - calc the total time needed for completing the release
# ----------------------------------------------------------------------------
sub calc_totaltime {
	open(FILLEDFILE, "< $filledfile") or return 0;
	my $starttime = 0;
	while(<FILLEDFILE>) {
		chomp($_);
		if (/(\d\d\d\d)-(\d\d)-(\d\d) (\d\d):(\d\d):(\d\d).+\.$/) {
	 		$starttime = stringtotime($_);
		}
	}
	close(FILLEDFILE);
	if ($starttime) { 
		return strdifftime($starttime, time); 
	} else { 
		return "unknown time"; 
	}
}


# ----------------------------------------------------------------------------
#  readdirectory() - read directory listing to a global array
# ----------------------------------------------------------------------------
sub read_directory {
	opendir(DIR, ".");
	my @files = grep {not /^\..*/} readdir(DIR);
	closedir(DIR);
	return @files;
}


# ----------------------------------------------------------------------------
#  sfv_exists() - does a sfv file exist ?
# ----------------------------------------------------------------------------
sub sfv_exists {
	opendir(DIR, ".");
	my @f = grep { /\.sfv$/i } readdir(DIR);
	closedir(DIR);
	$#f > -1 ? return 1 : return 0;
}


# ----------------------------------------------------------------------------
#  test_all() - check all files from an sfv file (if started with -T)
# ----------------------------------------------------------------------------
sub test_all {
	my %sfvhash = ();
	my $checked = 0;
	my $sfvfile;
	my ($file,$date, $size);
	my (@files, %filled, $filename, $fillstr);

	@files = read_directory();
	sysopen(FILLEDFILE, $filledfile, O_RDWR) or return 1;
	# LOCK_EX = 2
	flock(FILLEDFILE, 2) or return 1;
	$filled{'DIRECTORY'}=<FILLEDFILE>;
	while ($fillstr=<FILLEDFILE>) {
		chomp ($fillstr);
		($filename)=($fillstr =~ /^\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2} \S+\s+\S+\s+\d+\s+\d+\s+(.+)$/);
		$filled{$filename}=$fillstr;
	}
	print "primeCHECK (c) 2000 primemover\n";
	foreach $file (@files) {
		if ($file =~ /\.sfv$/i) {
			$sfvfile = $file;
			%sfvhash = read_sfvfile($sfvfile);
		}
		if ($file =~ /.*$missingsuffix$/) {
			unlink $file;
		}
	}
	if(!$sfvfile) {
		print " No sfvfile found !";
		exit 0;
	}
	print "Checking files against $sfvfile ...\n";
	# check all files
	my @keys = keys(%sfvhash);
	my $checksum;
	foreach $file (@keys) {
		if (-e $file) {
			if ( -e "$standarddir/bin/checksum") {
				$checksum = `$standarddir/bin/checksum "$file"`;
			} else {
			        die "Checksum util was not found and on the fly check is disabled.";
			}
		    
			chomp($checksum);
		    
			if ($checksum !~ /$sfvhash{$file}/i) {
				print "$file CORRUPT\n";
				rename $file, "$file.bad";
				delete $filled{$file};
				open(MISSINGFILE, "> $file$missingsuffix");
				close(MISSINGFILE);
			}
			else {
				if(!exists $filled{$file}) {
					($size,$date) = (stat($file))[7,9];
					$date=timetostring($date);
					$filled{$file}=sprintf("%s site       check        %10d     681984 %s",$date,$size,$file);
				}
				print "$file CRC OK\n";
				$__filename = $file;
				$checked += 1;
			}
		}
		else {
			print "$file MISSING\n";
			open(MISSINGFILE, "> $file$missingsuffix");
			close(MISSINGFILE);
		}
	}
	truncate (FILLEDFILE, 0) or return 1;
	# SEEK_SET = 0
	seek (FILLEDFILE, 0, 0) or return 1;
	print FILLEDFILE $filled{'DIRECTORY'};
	foreach $file (keys %filled) {
		if($file ne "DIRECTORY") {
			print FILLEDFILE $filled{$file}."\n";
		}
	}
	# LOCK_UN = 8
	flock(FILLEDFILE, 8);
	close(FILLEDFILE);
	@keys = keys(%sfvhash);
	if ($checked == $#keys + 1) {
		print "Release is complete!\n";
	} else {
		print "Release is incomplete. ($checked of ".($#keys+1)." files found.)\n";
	}
	$__permissionfile = ".permissions";
	status($checked,$#keys+1);
	$~ = "FOOTER"; write;
	exit 1;
}

# ----------------------------------------------------------------------------
#  rescan() - check all files uploaded before an sfv file 
# ----------------------------------------------------------------------------
sub rescan($$) {
	my($total, $sfvfilename) = @_;
	my $count = 0;
	my ($dummy, $crc, $fillstr, $filename);
	my %sfvlist = read_sfvfile($sfvfilename);

	if(-f $filledfile) {
		open(FILLED, "< $filledfile");
		while(<FILLED>) {
			$count++;
		}
		close(FILLED);
		$count--;
	}
	open(UNCHECKED, "< $unfilledfile");
	open(FILLED, ">> $filledfile");		# perhaps append is not the best thingy here ???
	while($dummy=<UNCHECKED>) {
		($crc,$fillstr)=($dummy =~ /(\S+)\s+(.+)/);
		($filename)=($fillstr =~ /^\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2} \S+\s+\S+\s+\d+\s+\d+\s+(.+)$/);
		if(uc($crc) eq uc($sfvlist{$filename})) {
			printf sprintf ("%-75s", " !C| CRC-POST-Check: $filename passed") ." |!0\n";
			printf FILLED "$fillstr\n";
			$count++;
		}
		else {
			rename $filename, "$filename.bad";
			printf sprintf ("%-75s", " !C| CRC-POST-Check: $filename FAILED") ." |!0\n";
		}
	}
	close(FILLED);
	close(UNCHECKED);
	unlink $unfilledfile;
	status($count,$total);
}

# ----------------------------------------------------------------------------
#   create_complete($totalfiles, $totalsize) - creates status file or dir
# ----------------------------------------------------------------------------
sub create_complete {
	my $totalfiles = shift;
	my $totalsize = shift;
	my $completemsg = $dircfg{'complete_dir'};
	my ($rank, $winner, $groupname, $files, $size, $speed, $percent);

	($rank, $winner, $groupname, $files, $size, $speed, $percent) = split(/:/, $racestats[0], 7);
	for ($completemsg) {
		s/\$totalfiles/$totalfiles/g;
		s/\$totalsize/$totalsize/g;
		s/\$totalracers/$totalracers/g;
		s/\$totalspeed/$totalspeed/g;
		s/\$winner/$winner/g;
	}
	if ($__filename =~ /.mp3$/i) {
		if (not %mp3info) { %mp3info = mp3_info($__filename); }
		for ($completemsg) {
			s/\$artist/$mp3info{'ARTIST'}/g;
			s/\$songname/$mp3info{'SONGNAME'}/g;
			s/\$album/$mp3info{'ALBUM'}/g;
			s/\$genre/$mp3info{'GENRE'}/g;
			s/\$year/$mp3info{'YEAR'}/g;
			s/\$comment/$mp3info{'COMMENT'}/g;
			s/\$bitrate/$mp3info{'BITRATE'}kbps/g;
			s/\$mode/$mp3info{'MODE'}/g;
			s/\$frequency/$mp3info{'FREQUENCY'}Hz/g;
			s/\$mpeg/$mp3info{'MPEG'}/g;
			s/\$layer/$mp3info{'LAYER'}/g;
			s/\$directory/$__adir/g;
		}
	}
	create($completemsg);
}

# ----------------------------------------------------------------------------
#    create($status) - creates an info-directory or file
# ----------------------------------------------------------------------------
sub create {
	my $status=$_[0];
	my $perm;
	if ($statustype == 1) {
		if ($del_status) {
		    $perm = pack "i12", 0, 0, "723723", 0, 0, 0, 0, 0, 0, 0, 0, 0;
		} else {
		    $perm = pack "i12", 0, 0, "197379", 0, 0, 0, 0, 0, 0, 0, 0, 0;
		}
		mkdir("$status", 0700);
		if (open PERMFILE, "> $status/$__permissionfile") {
			print PERMFILE $perm;
			close PERMFILE
		}
	}
	elsif ($statustype == 0) {
	    open(STATUS, "> $status");
	    close(STATUS);
	}
}

# ----------------------------------------------------------------------------
#    CHECK ZIP FILES
# ----------------------------------------------------------------------------
sub check_zip {
	my $filename = shift;
	my $statusfilepath;
	# check crc
	if (system("unzip -tqq $filename $shutup") != 0) {
		if (exists $dircfg{announce_badfile} and ($dircfg{announce_badfile} eq "yes")) {
			if ($debug) {
				print "DEBUG: badfile announce is enabled in this section!";
			}
			announce_badfile();
		}
		$message = "!BWARNING: file is corrupt or incomplete";
		$~ = "MESSAGE"; write;
		return 1;
	}
	# file is ok
	$message = "!Hfile passed the crc check successfully";
	$~ = "MESSAGE"; write;

	# extract file_id.diz
	if (system("unzip -Cpoqq $filename file_id.diz > file_id.diz") != 0) {
		$message = "!BWARNING: no file_id.diz was found ";
		$~ = "MESSAGE"; write;
		$statusfilepath = "..";
		if (exists $dircfg{statusfilepath}) {
			if (substr($dircfg{statusfilepath}, 0, 1) eq '/') { # absolute path
				$statusfilepath = $registry{basedir} . $dircfg{statusfilepath};
			}
			else {
				$statusfilepath = $dircfg{statusfilepath};
			}
		}
		open(SF, "> $statusfilepath/$__pdir/$__adir-NO.File_iD.DiZ" || printf "ERROR: Cant write statusfile\n");
		close(SF);
	}
	extract_nfo($filename);

	# don't count zips in sfv based releases
	if (sfv_exists() == 0) {
		write_filled($tm1, $__username, $__groupname, $filename, $__speed);
	}
	if (exists $dircfg{zipcommentfile} and -e $dircfg{zipcommentfile}) {
		if (system("zip -z -g -0 $filename < $dircfg{zipcommentfile} $shutup") != 0) {
			print "ERROR: Unable to add comment file to zip.\n";
		}
	}
	if (exists $dircfg{siteinfofile} and -e $dircfg{siteinfofile}) {
		if (system("zip -jgqq0 $filename $dircfg{siteinfofile}	$shutup") != 0) {
			print "ERROR: Unable to add file to zip.\n";
		}
	}

	my $goodfiles  = count_good();
	my $totalfiles = dizcount();

	if (is_new_racer()) {
		if ($debug) { print "DEBUG: new racer on zip release detected!"; }
		if (exists $dircfg{'announce_joinrace'} and ($dircfg{'announce_joinrace'} eq "yes")) {
			if ($debug) { print "DEBUG: joinrace announce is enabled in this section!"; }
			announce_joinrace();
		}
	}
	if (sfv_exists() == 0) {
		if ($debug) { print "DEBUG: No sfv file found, zip is counted!\n"; }
		status($goodfiles, $totalfiles);
	}
	return 0;
}

#############################################################################
# CHECK RAR OR MP3 FILES                                                    #
#############################################################################
sub check_rar {
	my $filename = shift;
	my %sfvhash = ();
	my $checksum;

	my @files = read_directory();

	# we got crc from the on-the-fly checker
	if (defined $__crc) {
		$checksum = $__crc;
	} 
	# we use the external checker
	else { 
		if ( -e "$standarddir/bin/checksum") {
			$checksum = `$standarddir/bin/checksum "$filename"`;
		} else {
			die "Checksum util was not found and on the fly check is disabled.";
		}
	}
    
	chomp($checksum);
	
	# search the sfv file
	foreach my $sfvfile (@files) {
		if ($sfvfile =~ /\.sfv$/i) {
			if ($debug) { print "DEBUG: sfv file found ($sfvfile)\n"; }
			%sfvhash = read_sfvfile($sfvfile);
		}
	}

	# file was not found in the sfv list ...
	if (not exists $sfvhash{lc($filename)}) {
		if (exists $dircfg{'enforce_sfv'} and $dircfg{'enforce_sfv'} eq "no") {
			$message = "!Hfile was accepted but not checked";
			$~ = "MESSAGE"; write;
			if($filename =~ $sfvtypes) {
				write_unfilled($checksum, $tm1, $__username, $__groupname, $filename, $__speed);
			}
	 		return 0;
		} else {
			$message = "!BWARNING: file was not found in sfv";
			$~ = "MESSAGE"; write;
			return 1;
		}
	}
    
	if($debug) {
		print "DEBUG: CRC from sfv: $sfvhash{$filename} - Len: ", length($sfvhash{lc($filename)}), "\n";
 		print "DEBUG: calced CRC  : $checksum - Len: ", length($checksum), "\n";
	}
	if ($checksum !~ /$sfvhash{lc($filename)}/i) {
		if (exists $dircfg{announce_badfile} and ($dircfg{announce_badfile} eq "yes")) {
			if ($debug) {
				print "DEBUG: badfile announce is enabled in this section!";
			}
			announce_badfile();
		}
		$message = "!BWARNING: file is corrupt or incomplete";
		$~ = "MESSAGE"; write;
		return 1;
	} else {
		$message = "!Hfile passed the crc check successfully";
		$~ = "MESSAGE"; write;
	}

	if (is_new_racer()) {
		if (exists $dircfg{'announce_joinrace'} and
	     	    ($dircfg{'announce_joinrace'} eq "yes")) {
    			if ($debug) { print "DEBUG: joinrace announce is enabled in this section!"; }
    			announce_joinrace();
 		}
	}
	write_filled($tm1, $__username, $__groupname, $filename, $__speed);
	my @keylist = keys(%sfvhash);
	my $totalfiles = $#keylist + 1;
	my $goodfiles  = count_good();

	# show id3info during race for mp3 files
	if ($filename =~ /mp3$/i && $id3_info_on_check) {
		%mp3info = mp3_info($filename);
		$~ = "LIMITER"; write;
		$~ = "MP3INFO"; write;
	}
	status($goodfiles, $totalfiles);
	return 0;
}

###############################################################################
# main program                                                                #
###############################################################################

sub main {
	$tm1 = timetostring(time);
	my $cwd = getcwd() or die "unable to get current working directory\n";
	# read registry to global hashes
	%registry = read_registry();

	$shutup = "2>/dev/null 1>&2";

	# check arguments
	if ($#ARGV < 0) {
		print "Syntax: primecheck.pl <filename> <transferspeed> [<checksum>] (if called from ftpd)\n";
		print "        primecheck.pl -T (if called from shell to test all files)\n\n";
		exit 1;
	}

	# test all files if program was started with -T flag
#	if ($ARGV[0] eq "-T") {
#		test_all();
#	}
	
	($__path, $__speed, $__crc) = @ARGV;

        # get absolute path
	if ($__path !~ /^\//) {
		$__path = "$cwd/$__path";
	}

	if (not $__speed or ($__speed !~ /\d+/)) {
		$__speed = -1;
	}

#	if (defined $debug) { 
#		print "DEBUG: [PATH = $__path] [SPEED = $__speed] [CRC = $__crc]\n"; 
#	}

	chdir(dirname($__path)) or die "ERROR: unable to find working directory\n";
	my @dirinfo = split(/\//, $__path);
	$__filename = $dirinfo[-1];

	##########################################################
	# special handling for cd1 style and sample dirs         #
	##########################################################
	if (($#dirinfo >= 2) and ($dirinfo[-2] =~ /^$special_dirs$/i)) {
		$__adir = $dirinfo[-3]." (".$dirinfo[-2].")";
		$__pdir = $dirinfo[-4];
		if ($link_status) {
			$linkdir = "/..";
			$linkpath = $dirinfo[-3] . "/" . $dirinfo[-2];
		}
	} else {
		$__adir = $dirinfo[-2];
		$__pdir = $dirinfo[-3];
		if ($link_status) {
			$linkdir = "/.";
			$linkpath = $dirinfo[-2];
		}
	}

	if ($debug) { print "DEBUG: announcedir: $__adir\n"; }

	my @files = read_directory();

	# read the openftpd username and groupname from environment variables
	# and store them in global vars
	if (exists $ENV{'F4A_USER'}) { 
		$__username = $ENV{'F4A_USER'};
	} else { 
		$__username = "unknown";
	}
	
	if (exists $ENV{'F4A_GROUP'}) { 
		$__groupname = $ENV{'F4A_GROUP'};
	} else { 
		$__groupname = "unknown";
	}

	# read section config to global hashes
	%dircfg   = read_dircfg($cwd);

	# test all files if program was started with -T flag
	if ($ARGV[0] eq "-T") {
		if (exists $registry{shortsitename}) {
			$__sitename = $registry{'shortsitename'};
		}
		else {
			$__sitename =" ";
		}
		test_all();
	}

	##########################################################
	# get some data from registry                            #
	##########################################################
	if (exists $registry{'botpidfile'}) {
		$__botpidfile = $registry{'botpidfile'};
	}
	if (exists $registry{'announcefile'}) {
		$__announcefile = $registry{'announcefile'};
	}

	$__permissionfile = ".permissions";
	if (exists $registry{permissionfile}) {
		$__permissionfile = $registry{'permissionfile'};
	}
	if (exists $registry{shortsitename}) {
		$__sitename = $registry{'shortsitename'};
	}

	$~ = "HEADER"; write;


	##########################################################
	# check for illegal chars                                #
	##########################################################
	if ($__filename =~ /[;&\$\/\`]/) {
	    #`
	    $message = "!BERROR: illegal meta characters in the filename!";
	    $~ = "MESSAGE"; write;
	    return 1;
	}

	##########################################################
	# check if filetype is allowed                           #
	##########################################################
	if (exists $dircfg{'allow_check'}) {
		if (($dircfg{'allow_check'} =~ /all except +(.+)/i and $__filename =~ /$1/i)
			or ($dircfg{'allow_check'} !~ /all except +(.+)/i and
			$__filename !~ /$dircfg{'allow_check'}/i)) {
				$message = "!BWARNING: this type of file is not allowed in this directory";
				$~ = "MESSAGE"; write;
				return 1;
		}
	} else {
		if ($debug) { print "DEBUG: key allow_check was not found!\n"; }
		$message = "!BWARNING: this type of file is not allowed in this directory";
		$~ = "MESSAGE"; write;
		return 1;
	}


	##########################################################
	# checksum files in sfv format                           #
	##########################################################
	if ($__filename =~ /\.sfv$/i) {
	#	$m3u_name = $__filename =~ s/\.sfv/.\m3u/g;
		if ($debug) { print "DEBUG: sfv file detected.\n"; }
		my %sfvhash = read_sfvfile($__filename);
		my @keylist = keys(%sfvhash);
		create_missing(@keylist);
		status(0, $#keylist + 1);
		if (-e $unfilledfile) {
			rescan($#keylist+1,$__filename);
		}
	}

	##########################################################
	# sample/*.avi -> announce_sample ($res,$frames,$codec)  #
	##########################################################
	elsif ($__path =~ /sample\/.+\.avi$/i) {
	    if ((exists $dircfg{'announce_sample'}) and ($dircfg{'announce_sample'} eq 'yes')) {
	        if ($debug) { print "DEBUG: avi file detected.\n"; }
	        announce_sample();
	    }
	    return 0;
	}

	##########################################################
	# zip files                                              #
	##########################################################
	elsif ($__filename =~ /\.zip$/i) {
		if ($debug) { print "DEBUG: zip file detected.\n"; }
		return check_zip($__filename);
	}

	##########################################################
	# filetypes that will be validated against sfv files     #
	##########################################################
	elsif ($__filename =~ /$sfvtypes$/i) {
		if ($debug) { print "DEBUG: rar or mp3 file detected.\n"; }
		return check_rar($__filename);
	}
	# nfo file
	elsif ($__filename =~ /\.nfo$/i) {
		if (exists $dircfg{'announce_imdb'} and $dircfg{'announce_imdb'} eq 'yes') {
			my $url = get_imdb_url($__path);
			$__imdb_url = $url;
			if ($url =~ 'imdb.com') {
				$message = "Requesting movie information from IMDB ..";
				$~ = "MESSAGE"; write;
				%imdb = parse_imdb($url);
				announce_imdb;
			}
		}
		if (exists $dircfg{'dccinfo'} and $dircfg{'dccinfo'} eq 'yes') {
			system ('cp',$__filename,$dircfg{dccpath}."/$__adir\_$__filename");
			announce_dccinfo($__filename);
		}
		if (exists $dircfg{'announce_http'} and $dircfg{'announce_http'} eq 'yes') {
			$__http_url = get_http_url($__path);
			if ($__http_url !~ "--") {
				announce_http();
			}
		}
		return 0;
	}

	##########################################################
	# unknown but allowed filetype                           #
	##########################################################
	else {
	   	$message = "!Hfile was accepted but not checked";
	   	$~ = "MESSAGE"; write;
	}
	return 0;
}

my $rcode = main();
$~ = "FOOTER"; write;
exit($rcode);

# --- end of file ---
