#!/usr/bin/perl -w

##############################################################################
#                                                                            #
#  OpenFTPD Request Script v1.0 - (c) 2000 topace & primemover               #
#                                                                            #
# ---------------------------------------------------------------------------#
#                                                                            #
#  This program is free software; you can redistribute it and/or modify      #
#  it under the terms of the GNU General Public License as published by      #
#  the Free Software Foundation; either version 2 of the License, or         #
#  (at your option) any later version.                                       #
#                                                                            #
##############################################################################

my $standarddir   = $ENV{'F4A_STD'};
my $requestlog    = $standarddir."/var/log/request.log";
my $header        = $standarddir."/msg/req_top";
my $footer        = $standarddir."/msg/req_tail";
my $rotatedays    = 21;

use strict; #always
use FindBin;
use lib "$FindBin::Bin/";
use primetools;

our ($sitename,$requestdir, $username, $groupname,$announcefile, $botpidfile);

# ----------------------------------------------------------------------------
#   announce_request() - what do you think is this doing ? 
# ----------------------------------------------------------------------------
sub announce_request {
   my $formatfile = $standarddir."msg/irc/request";
   my $formatstring = "";

   open(FORMATFILE, "< $formatfile") or return 0;
   chomp($formatstring = <FORMATFILE>);
   close(FORMATFILE);

   (my $minutes, my $hours) = (localtime)[1..2];
   my $time = sprintf("%02d:%02d", $hours, $minutes);

   for ($formatstring) {
      s/<b>|<\/b>/$b/g;
      s/<u>|<\/u>/$u/g;
      s/\$sitename/$sitename/g;
      s/\$directory/$requestdir/g;
      s/\$username/$username/g;
      s/\$groupname/$groupname/g;
      s/\$time/$time/g;
   }
   announce_line($announcefile, $botpidfile, $formatstring);
}

sub timetostring_local {
      my ($day_of_month, $month, $year) = (localtime)[3..5];
   my $timestring = sprintf("%04d-%02d-%02d",  $year+1900, $month+1,
   $day_of_month);
   return $timestring;
}

sub dayssince {
  my ($day_of_month, $month, $year) = (localtime)[3..5];
  my ($thenyear,$thenmonth,$thenday)=split("-",$_[0],3); 
  my $days;
  if($day_of_month >= $thenday ) { 
     $days=$day_of_month-$thenday;
  }
  else {
    #ok i know this is dumb and will cause requests to stay for an extra day
    #or be deleted a day early sometimes, but i dont know an easy way of
    #finding the number of days in a  month.  if someone else can, just replace
    #the 30 with a  daysin($thenmonth) method or whatever.  
    $days=30+$day_of_month-$thenday;
  } 
  
  return $days;
 } 

###############################################################################
# main program                                                                #
###############################################################################
my $needwrite=0;
my @requests;
my @newrequests;
# show existing requests
if ($#ARGV < 0 ) {
   if (-s $header and -s $footer) { 
      print `cat $header`;
      if (open (REQFILE, "< $requestlog")) {
	  while (<REQFILE>) {
             chomp($_);
             if ($_) { push(@requests, $_); }
	  }
	  close(REQFILE);
      }
      my $num=0;
      foreach my $line (@requests) {
         my ($timestamp, $username, $groupname, $dir) = split(/\s+/, $line, 4);
         my $numdays=dayssince($timestamp);
         # put each line into a new hash, so we can re-write the file if soem entries have expired
         if($numdays < $rotatedays) {
           printf (" !C| !H%2d  %-9.9s %-10.10s  %-44.44s !C|!0\n", $num, $username, $timestamp, $dir);
           push(@newrequests, sprintf("%s %-10s %-10s %s\n", $timestamp, $username, $groupname, $dir));
         }
         else {
           #indicate that some entries have expired, so we need to re-write the request.log
           $needwrite=1;
         }
         $num++;
      }
      print `cat $footer`;
   }
   if( $needwrite eq 1) {
      open(REQUESTLOG,"> $requestlog") or die "ERROR: unable to open request log";
      foreach $_ (@newrequests) {
        printf REQUESTLOG ;
      }
      close(REQUESTLOG);
   }
}
# add a request
else {
   # read the username and groupname from environment variables
   if (exists $ENV{'F4A_USER'}) { $username = $ENV{'F4A_USER'} }
   else { $username = "unknown" };
   if (exists $ENV{'F4A_GROUP'}) { $groupname = $ENV{'F4A_GROUP'} }
   else { $groupname = "unknown" };
   #read registry and dircfg to global hashes
   my %registry = read_registry(); 
   if (exists $registry{botpidfile}) { 
      $botpidfile = $registry{botpidfile}; 
   }
   if (exists $registry{announcefile}) { 
      $announcefile = $registry{announcefile}; 
   }
   $requestdir = "@ARGV";
   #get rid of spaces
   $requestdir =~ s/ /_/g;

   if (exists $registry{shortsitename}) {
      $sitename = $registry{shortsitename};
   }
   open(REQUESTLOG,">> $requestlog") or 
      die "ERROR: unable to open request log";
   printf REQUESTLOG "%s %-10s %-10s %s\n", timetostring_local(), $username,
      $groupname, $requestdir;
   close(REQUESTLOG);
   announce_request();
   print " Request successfully made.\n";
}
exit 0;

# --- end of file ---
